// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ErrorManager.cpp
//
// $Id: ErrorManager.cpp,v 1.3 2002/02/26 06:59:40 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "ErrorManager.h"

#ifdef HAVE_EXECINFO_H
#include <execinfo.h>
#endif

#include <stdio.h>


#include "ErrorDialog.h"
#include "Log.h"

namespace xb {

//------------------------------------------------------------------------------

ErrorManager* ErrorManager::s_pInstance = NULL;

//------------------------------------------------------------------------------

ErrorManager::ErrorManager()
{

}

//------------------------------------------------------------------------------

ErrorManager*
ErrorManager::I()
{
	if (s_pInstance == NULL) {
		s_pInstance = new ErrorManager;
	}

	return s_pInstance;
}

//------------------------------------------------------------------------------

ostream&
ErrorManager::Noise() const
{
	return m_Log.Noise();
}

//------------------------------------------------------------------------------

ostream&
ErrorManager::Trace() const
{
	return m_Log.Trace();
}

//------------------------------------------------------------------------------

ostream&
ErrorManager::Warn() const
{
	return m_Log.Warn();
}

//------------------------------------------------------------------------------

ostream&
ErrorManager::Error() const
{
	return m_Log.Error();
}

//------------------------------------------------------------------------------

void
ErrorManager::SetLogLevel(LogLevel nLevel)
{
	m_Log.SetLevel((LogStream::LogLevel)nLevel);
}

//------------------------------------------------------------------------------

void
ErrorManager::Dialog(const std::string& strMessage)
{
	Error() << strMessage << endl;
	
	ErrorDialog::Instance()->SetText(strMessage);
	ErrorDialog::Instance()->CenterOverDisplay();
	ErrorDialog::Instance()->show_all();
}

//------------------------------------------------------------------------------

void
ErrorManager::BackTrace() const
{
#ifdef HAVE_EXECINFO_H	
	void* buf[10];
	int count = backtrace(buf, 10);
	char** s = backtrace_symbols(buf, count);
	for (int i = 0; i < count; i++) {
		Error() << s[i] << endl;
    }
#else
	Error() << "no backtrace function available" << endl;
#endif
}

//------------------------------------------------------------------------------

file_error::file_error(const std::string& str) : runtime_error(str)
{
	EM::I()->BackTrace();
}

//------------------------------------------------------------------------------

};

//------------------------------------------------------------------------------
