// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ErrorManager.h
//
// $Id: ErrorManager.h,v 1.3 2002/02/26 06:59:40 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __ErrorManager__
#define __ErrorManager__

#include <iostream>
#include <stdexcept>
#include <string>

#include "Error.h"				// assert stuff
#include "LogStream.h"

namespace xb {

//------------------------------------------------------------------------------

#define FILELINE __FILE__ << " [" << __LINE__ << "]: "	

//------------------------------------------------------------------------------

class ErrorManager
{
	// Constants
	public:
		enum LogLevel {
			NOISE,
			TRACE,
			WARNING,
			ERROR,
			NONE
		};
		
	// Types
	// Variables
	private:
		static ErrorManager* s_pInstance;
		
		LogStream m_Log;
		
	// Methods
	private:
		ErrorManager();

	public:
		static ErrorManager* I();

		// Log functions
		/*
	    void Noise(const std::string& strMessage) const; 
		void Trace(const std::string& strMessage) const;
		void Warn(const std::string& strMessage) const;
		void Error(const std::string& strMessage) const;
		*/
		void SetLogLevel(LogLevel nLevel);

		ostream& Noise() const;
		ostream& Trace() const;
		ostream& Warn() const;
		ostream& Error() const;
		
		// Dialog functions
		void Dialog(const std::string& strMessage);

		void BackTrace() const;
};

typedef ErrorManager EM;


class file_error : public runtime_error
{
	public:
		file_error(const std::string& str);
};

//------------------------------------------------------------------------------

};

#endif 

//------------------------------------------------------------------------------
