// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Layer.h
//
// $Id: Layer.h,v 1.2 2002/01/27 02:12:48 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __Layer__
#define __Layer__

#include "Frame.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class BSBuffer;
class PlayerInfo;
class Output;
class Visualization;

class Layer
{
	// Constants
	public:
		static const int SBLIMIT = 32;
		static const int SSLIMIT = 18;
		static const int MPG_MD_STEREO = 0;
		static const int MPG_MD_JOINT_STEREO = 1;
		static const int MPG_MD_DUAL_CHANNEL = 2;
		static const int MPG_MD_MONO = 3;
		static const int SCALE_BLOCK = 12;
		
	// Types
		
	// Variables
	protected:
		PlayerInfo* m_pInfo;
		Output* m_pOutput;
		Visualization* m_pVisualization;

		unsigned char* m_pPCMSample;
		int* m_pnPCMPoint;

		BSBuffer* m_pBuffer;
		
		static unsigned long rval;
		static unsigned char rval_uc;

		float mpg123_muls[27][64];

		int m_nLevel;

	// Methods
	public:
		Layer(PlayerInfo* pInfo, Output* pOutput, Visualization* pVis,
			  unsigned char* pSample, int* pPoint, BSBuffer* pBuffer);
		
		virtual ~Layer();

		virtual int Run(Frame* fr) = 0;

		virtual void Init(int nVal) = 0;
		
		int GetLevel() { return m_nLevel; }
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
