// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Layer1.cpp
//
// $Id: Layer1.cpp,v 1.4 2002/02/26 19:30:26 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "Layer1.h"

#include <xb/ErrorManager.h>

#include "BSBuffer.h"
#include "Output.h"
#include "PlayerInfo.h"
#include "Synth.h"
#include "Thread.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

Layer1::Layer1(PlayerInfo* pInfo, Output* pOutput, Visualization* pVis,
			   unsigned char* pSample, int* pPoint, BSBuffer* pBuffer) :
	Layer(pInfo, pOutput, pVis, pSample, pPoint, pBuffer)
{
	m_nLevel = 1;
}

//------------------------------------------------------------------------------

Layer1::~Layer1()
{

}

//------------------------------------------------------------------------------

int
Layer1::Run(Frame* fr)
{
	EM::I()->Trace() << "PlayerMPG::DoLayer1" << endl;

	int i, stereo = fr->stereo;
	unsigned int balloc[2 * SBLIMIT];
	unsigned int scale_index[2][SBLIMIT];
	float fraction[2][SBLIMIT];
	int single = fr->single;

	fr->jsbound = (fr->mode == MPG_MD_JOINT_STEREO) ? (fr->mode_ext << 2) + 4 : 32;

	if (stereo == 1 || single == 3)
		single = 0;

	I_step_one(balloc, scale_index, fr);

	for (i = 0; i < SCALE_BLOCK; i++)
	{
		I_step_two(fraction, balloc, scale_index, fr);

		if (single >= 0)
		{
			// PATRICK: fixme
			// fr->synth_mono->Run((float *) fraction[single], m_pPCMSample,
			// m_pnPCMPoint);
		}
		else
		{
			int p1 = *m_pnPCMPoint;

			fr->synth->Run((float *) fraction[0], 0, m_pPCMSample, &p1);
			fr->synth->Run((float *) fraction[1], 1, m_pPCMSample,
						   m_pnPCMPoint);
		}

		if (m_pInfo->output_audio)
		{
			// PATRICK: fixme
			/*
			mpg123_ip.add_vis_pcm(mpg123_ip.output->written_time(),
								  mpg123_cfg.resolution == 16 ? FMT_S16_NE : FMT_U8,
								  mpg123_cfg.channels == 2 ? fr->stereo : 1, m_nPCMPoint,
								  m_pPCMSample);
			*/

			while (m_pOutput->BufferFreeAmt() < *m_pnPCMPoint &&
				   m_pInfo->going &&
				   m_pInfo->jump_to_time == -1) {
				Thread::usleep(10000);
			}

			if (m_pInfo->going && m_pInfo->jump_to_time == -1) {
				m_pOutput->Write(m_pPCMSample, *m_pnPCMPoint);
			}
		}
		*m_pnPCMPoint = 0;
	}

	return 1;
}

//------------------------------------------------------------------------------

void
Layer1::I_step_one(unsigned int balloc[],
				   unsigned int scale_index[2][SBLIMIT], Frame *fr)
{
	unsigned int *ba = balloc;
	unsigned int *sca = (unsigned int *) scale_index;

	if (fr->stereo)
	{
		int i;
		int jsbound = fr->jsbound;

		for (i = 0; i < jsbound; i++)
		{
			*ba++ = m_pBuffer->GetBits(4);
			*ba++ = m_pBuffer->GetBits(4);
		}
		for (i = jsbound; i < SBLIMIT; i++)
			*ba++ = m_pBuffer->GetBits(4);

		ba = balloc;

		for (i = 0; i < jsbound; i++)
		{
			if ((*ba++))
				*sca++ = m_pBuffer->GetBits(6);
			if ((*ba++))
				*sca++ = m_pBuffer->GetBits(6);
		}
		for (i = jsbound; i < SBLIMIT; i++)
			if ((*ba++))
			{
				*sca++ = m_pBuffer->GetBits(6);
				*sca++ = m_pBuffer->GetBits(6);
			}
	}
	else
	{
		int i;

		for (i = 0; i < SBLIMIT; i++)
			*ba++ = m_pBuffer->GetBits(4);
		ba = balloc;
		for (i = 0; i < SBLIMIT; i++)
			if ((*ba++))
				*sca++ = m_pBuffer->GetBits(6);
	}
}

//------------------------------------------------------------------------------

void
Layer1::I_step_two(float fraction[2][SBLIMIT],
					  unsigned int balloc[2 * SBLIMIT],
					  unsigned int scale_index[2][SBLIMIT], Frame *fr)
{
	int i, n;
	int smpb[2 * SBLIMIT];	/* values: 0-65535 */
	int *sample;
	register unsigned int *ba;
	register unsigned int *sca = (unsigned int *) scale_index;

	if (fr->stereo)
	{
		int jsbound = fr->jsbound;
		register float *f0 = fraction[0];
		register float *f1 = fraction[1];

		ba = balloc;
		for (sample = smpb, i = 0; i < jsbound; i++)
		{
			if ((n = *ba++))
				*sample++ = m_pBuffer->GetBits(n + 1);
			if ((n = *ba++))
				*sample++ = m_pBuffer->GetBits(n + 1);
		}
		for (i = jsbound; i < SBLIMIT; i++)
			if ((n = *ba++))
				*sample++ = m_pBuffer->GetBits(n + 1);

		ba = balloc;
		for (sample = smpb, i = 0; i < jsbound; i++)
		{
			if ((n = *ba++))
				*f0++ = (float) (((-1) << n) + (*sample++) + 1) * mpg123_muls[n + 1][*sca++];
			else
				*f0++ = 0.0;
			if ((n = *ba++))
				*f1++ = (float) (((-1) << n) + (*sample++) + 1) * mpg123_muls[n + 1][*sca++];
			else
				*f1++ = 0.0;
		}
		for (i = jsbound; i < SBLIMIT; i++)
		{
			if ((n = *ba++))
			{
				float samp = (((-1) << n) + (*sample++) + 1);

				*f0++ = samp * mpg123_muls[n + 1][*sca++];
				*f1++ = samp * mpg123_muls[n + 1][*sca++];
			}
			else
				*f0++ = *f1++ = 0.0;
		}
		for (i = fr->down_sample_sblimit; i < 32; i++)
			fraction[0][i] = fraction[1][i] = 0.0;
	}
	else
	{
		register float *f0 = fraction[0];

		ba = balloc;
		for (sample = smpb, i = 0; i < SBLIMIT; i++)
			if ((n = *ba++))
				*sample++ = m_pBuffer->GetBits(n + 1);
		ba = balloc;
		for (sample = smpb, i = 0; i < SBLIMIT; i++)
		{
			if ((n = *ba++))
				*f0++ = (float) (((-1) << n) + (*sample++) + 1) * mpg123_muls[n + 1][*sca++];
			else
				*f0++ = 0.0;
		}
		for (i = fr->down_sample_sblimit; i < 32; i++)
			fraction[0][i] = 0.0;
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
