// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// PlayerMPG.h
//
// $Id: PlayerMPG.h,v 1.12 2002/02/18 09:21:57 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __PlayerMPG__
#define __PlayerMPG__

#include <glib.h>
#include <stdio.h>

#include "Player.h"
#include "Thread.h"

#include "BSBuffer.h"
#include "Frame.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class Layer;
class Output;
class PlayerInfo;
class Visualization;

class PlayerMPG : public Player, Thread
{
	// Constants
	private:
		
		static const int SBLIMIT = 32;

	// Types
	private:
		//  XHEADDATA Copyright 1998 Xing Technology Corp.
		// 
		struct XHEADDATA
		{
				int h_id;               /* from MPEG header, 0=MPEG2, 1=MPEG1 */
				int samprate;           /* determined from MPEG header */
				int flags;              /* from Xing header data */
				int frames;             /* total bit stream frames from Xing header data
										 */
				int bytes;              /* total bit stream bytes from Xing header data 
										 */
				int vbr_scale;          /* encoded vbr scale from Xing header data */
				unsigned char *toc;     /* pointer to unsigned char toc_buffer[100] */
				/* may be NULL if toc not desired */ 
		};

	// Variables

	private:
		int m_nResolution;
		int m_nChannels;
		int m_nDownsample;
		int m_nDownsampleCustom;

		Frame m_Frame;
		Frame m_FrameTemp;

		PlayerInfo* m_pInfo;

		int m_nSkipFrames;
		bool m_bAudioError;
		bool m_bHaveXingHeader;

		xb::Filename m_strFilename;

		// Output* m_pOutput;
		// Visualization* m_pVisualization;

		int m_nBitRate;

		unsigned char* m_pPCMSample;
		int m_nPCMPoint;

		BSBuffer m_BSBuffer;
		
		FILE* m_pFile;

		int m_nLSF;
		bool m_bMpeg25;
		int m_nStereo;
		int m_nDispBitRate;
		int m_nMode;

		int m_nLength;

		int m_nFrequency;
		int m_nLayer;

		unsigned char m_acXingTOC[100];
		XHEADDATA xing_header;

		Layer* m_pLayer;

		int m_nTrack;
	
    // Methods
	public:
		PlayerMPG();
		~PlayerMPG();

		virtual void Init(Output* pOutput, Visualization* pVis);

		virtual void Play(const xb::Filename& strFilename, int nTrack);
		virtual void Stop(bool bSendSignal = true);
		virtual void Pause(bool bPause);
		virtual void Seek(int nTime);
		virtual void SeekPercentage(float fPercent);
		virtual int GetTime() const;
		
		virtual bool CanPlayFile(const xb::Filename& strFilename);

		virtual bool CheckCapability(std::string& strMessage);

		virtual void Run();
		
	private:
		void ReadFrameInit();
		void OpenStream();
		int StreamInit();
		int GetFileInfo();
		int FullRead(FILE* pFile, unsigned char* pBuf, int nCount);
		bool ReadFrame(Frame* pFrame);
		bool StreamHeadRead(unsigned long* newhead);
		bool HeadCheck(unsigned long head);

		bool ReadID3v2Tag(unsigned long head);
		int StreamHeadShift(unsigned long* pHead);
		int StreamReadFrameBody(unsigned char* buf, int size);

		void SetSynthFunctions(Frame* fr);

		int CalcNumFrames(Frame* fr);

		void PlayFrame(Frame* fr);

		void StreamJumpToFrame(Frame* fr, int nFrame);
		void StreamJumpToByte(Frame* fr, int nByte);
		void StreamClose();
		
		int SeekPoint(unsigned char TOC[100], int file_bytes, float percent);

		void Dump();
		static std::string BoolStr(bool bIn);
		
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------



