// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Synth1to1.cpp
//
// $Id: Synth1to1.cpp,v 1.3 2002/01/26 03:23:25 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "Synth1to1.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

Synth1to1::Synth1to1()
{

}

//------------------------------------------------------------------------------

Synth1to1::~Synth1to1()
{

}

//------------------------------------------------------------------------------

#define WRITE_SAMPLE(samples,sum,clip) { \
  double dtemp; int v; /* sizeof(int) == 4 */ \
  dtemp = ((((65536.0 * 65536.0 * 16)+(65536.0 * 0.5))* 65536.0)) + (sum);  \
  v = ((*(int *)&dtemp) - 0x80000000); \
  if( v > 32767) { *(samples) = 0x7fff; (clip)++; } \
  else if( v < -32768) { *(samples) = -0x8000; (clip)++; } \
  else { *(samples) = v; }  \
}

//------------------------------------------------------------------------------

int
Synth1to1::Run(float* bandPtr, int channel, unsigned char* out, int* pnt)
{
	static float buffs[2][2][0x110];
	static const int step = 2;
	static int bo = 1;
	short *samples = (short *) (out + *pnt);

	float *b0, (*buf)[0x110];
	int clip = 0;
	int bo1;

	if (!channel)
	{
		bo--;
		bo &= 0xf;
		buf = buffs[0];
	}
	else
	{
		samples++;
		buf = buffs[1];
	}

	if (bo & 0x1)
	{
		b0 = buf[0];
		bo1 = bo;
		mpg123_dct64(buf[1] + ((bo + 1) & 0xf), buf[0] + bo, bandPtr);
	}
	else
	{
		b0 = buf[1];
		bo1 = bo + 1;
		mpg123_dct64(buf[0] + bo, buf[1] + bo + 1, bandPtr);
	}

	{
		register int j;
		float *window = m_apDecwin + 16 - bo1;

		for (j = 16; j; j--, b0 += 0x10, window += 0x20, samples += step)
		{
			float sum;

			sum = window[0x0] * b0[0x0];
			sum -= window[0x1] * b0[0x1];
			sum += window[0x2] * b0[0x2];
			sum -= window[0x3] * b0[0x3];
			sum += window[0x4] * b0[0x4];
			sum -= window[0x5] * b0[0x5];
			sum += window[0x6] * b0[0x6];
			sum -= window[0x7] * b0[0x7];
			sum += window[0x8] * b0[0x8];
			sum -= window[0x9] * b0[0x9];
			sum += window[0xA] * b0[0xA];
			sum -= window[0xB] * b0[0xB];
			sum += window[0xC] * b0[0xC];
			sum -= window[0xD] * b0[0xD];
			sum += window[0xE] * b0[0xE];
			sum -= window[0xF] * b0[0xF];

			WRITE_SAMPLE(samples, sum, clip);
		}

		{
			float sum;

			sum = window[0x0] * b0[0x0];
			sum += window[0x2] * b0[0x2];
			sum += window[0x4] * b0[0x4];
			sum += window[0x6] * b0[0x6];
			sum += window[0x8] * b0[0x8];
			sum += window[0xA] * b0[0xA];
			sum += window[0xC] * b0[0xC];
			sum += window[0xE] * b0[0xE];
			WRITE_SAMPLE(samples, sum, clip);
			b0 -= 0x10, window -= 0x20, samples += step;
		}
		window += bo1 << 1;

		for (j = 15; j; j--, b0 -= 0x10, window -= 0x20, samples += step)
		{
			float sum;

			sum = -window[-0x1] * b0[0x0];
			sum -= window[-0x2] * b0[0x1];
			sum -= window[-0x3] * b0[0x2];
			sum -= window[-0x4] * b0[0x3];
			sum -= window[-0x5] * b0[0x4];
			sum -= window[-0x6] * b0[0x5];
			sum -= window[-0x7] * b0[0x6];
			sum -= window[-0x8] * b0[0x7];
			sum -= window[-0x9] * b0[0x8];
			sum -= window[-0xA] * b0[0x9];
			sum -= window[-0xB] * b0[0xA];
			sum -= window[-0xC] * b0[0xB];
			sum -= window[-0xD] * b0[0xC];
			sum -= window[-0xE] * b0[0xD];
			sum -= window[-0xF] * b0[0xE];
			sum -= window[-0x0] * b0[0xF];

			WRITE_SAMPLE(samples, sum, clip);
		}
	}
	*pnt += 128;

	return clip;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
