// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ActiveList.cpp
//
// $Id: ActiveList.cpp,v 1.32 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include "ActiveList.h"

#include <list>
#include <xb/ErrorManager.h>

#include "InterfaceTypes.h"
#include "PropList.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

ActiveList::ActiveList() :
	xb::XCList(5),
	m_PMCboxNoAudioEmpty(INTERFACE_DATA_DIR "/cbox-noaudio-empty.xpm"),
	m_PMCboxAudioEmpty(INTERFACE_DATA_DIR "/cbox-audio-empty.xpm"),
	m_PMCboxNoAudioFull(INTERFACE_DATA_DIR "/cbox-noaudio-full.xpm"),
	m_PMCboxAudioFull(INTERFACE_DATA_DIR "/cbox-audio-full.xpm"),
	m_ColorOdd("#edf3ff"),
	// m_ColorOdd("#292c29"),
	// m_ColorEven("#081018"),
	m_ColorEven("#ffffff"),
	m_bReorderable(false),
	m_nSortColumn(0),
	m_nSortType(GTK_SORT_ASCENDING),
	m_bSongPlaying(false)
	
{
	column_titles_show();
	
	column(0).set_resizable(false);
	column(0).set_auto_resize(true);
	column(0).set_title("");

	column(1).set_title("Song");
	column(1).set_resizable(true);
	column(1).set_width(System::GetPropList()->GetIntValue("SongColumnWidth"));

	column(2).set_title("Time");
	column(2).set_resizable(false);
	column(2).set_auto_resize(true);

	column(3).set_title("Artist");
	column(3).set_resizable(true);
	column(3).set_width(System::GetPropList()->GetIntValue("ArtistColumnWidth"));

	column(4).set_title("Album");
	column(4).set_resizable(true);
	column(4).set_width(System::GetPropList()->GetIntValue("AlbumColumnWidth"));
	
	set_usize(300, 200);
	set_shadow_type(GTK_SHADOW_ETCHED_IN);

	set_selection_mode(GTK_SELECTION_EXTENDED);

	SetReorderable(m_bReorderable);

	drag_data_get.connect(SigC::slot(this, &ActiveList::DNDDataGet));

	// button 1 reserved for set_reorderable...
	drag_source_set(GDK_BUTTON2_MASK,
					InterfaceTypes::s_aTargetTableStrings,
					1,
					(GdkDragAction)GDK_ACTION_COPY);

	// looks like you can't call this more than once...
	/*
	drag_source_set(GDK_BUTTON2_MASK,
					InterfaceTypes::s_aTargetTableRownums,
					1,
					(GdkDragAction)GDK_ACTION_MOVE);
	*/


	drag_data_received.connect(SigC::slot(this, &ActiveList::DNDReceive));
	drag_data_delete.connect(SigC::slot(this, &ActiveList::DNDDelete));
	

	click_column.connect(SigC::slot(this, &ActiveList::ColumnClick));
	
	set_use_drag_icons(true);

	column(0).get_widget()->set_name("clistbutton");
}

//------------------------------------------------------------------------------

ActiveList::~ActiveList()
{
	ClearData();
}

//------------------------------------------------------------------------------

void
ActiveList::AppendSong(const Song& song)
{
	freeze();
	
	std::list<std::string> listColumns;
	listColumns.push_back(song.GetTrack());
	listColumns.push_back(song.GetTitle());
	listColumns.push_back(song.GetTime());
	listColumns.push_back(song.GetArtist());
	listColumns.push_back(song.GetAlbum());

	rows().push_back(listColumns);
	Song* pSong = new Song(song);
	rows().back().set_data(pSong);
	

	if (rows().size() % 2 == 1) {
		rows().back().set_background(m_ColorOdd);
	}
	else {
		rows().back().set_background(m_ColorEven);
	}

	thaw();
}

//------------------------------------------------------------------------------

void
ActiveList::SetCheckbox(unsigned int nRow, bool bOn)
{
	Assert(nRow >= 0, "nRow < 0");
	Assert(nRow < rows().size(), "nRow >= rows().size");

	if (bOn) {
		cell(nRow, 0).set_pixmap(m_PMCboxNoAudioFull.GetPixmap(),
								 m_PMCboxNoAudioFull.GetMask());
	}
	else {
		cell(nRow, 0).set_pixmap(m_PMCboxNoAudioEmpty.GetPixmap(),
								 m_PMCboxNoAudioEmpty.GetMask());
	}

	(GetSongPtr(nRow))->SetSelected(bOn);
}

//------------------------------------------------------------------------------

void
ActiveList::SetAudio(unsigned int nRow, bool bOn)
{
	// PATRICK: efence has problems with this fn
	
	Assert(nRow >= 0, "nRow < 0");
	Assert(nRow < rows().size(), "nRow >= rows().size");

	bool bSelected = IsSelected(nRow);

	if (bSelected) {

		if (bOn) {
			cell(nRow, 0).set_pixmap(m_PMCboxAudioFull.GetPixmap(),
									 m_PMCboxAudioFull.GetMask());
		}
		else {
			cell(nRow, 0).set_pixmap(m_PMCboxNoAudioFull.GetPixmap(),
									 m_PMCboxNoAudioFull.GetMask());
		}
		
	}
	else {

		if (bOn) {
			cell(nRow, 0).set_pixmap(m_PMCboxAudioEmpty.GetPixmap(),
									 m_PMCboxAudioEmpty.GetMask());
		}
		else {
			cell(nRow, 0).set_pixmap(m_PMCboxNoAudioEmpty.GetPixmap(),
									 m_PMCboxNoAudioEmpty.GetMask());
		}
		
	}

	
	Song* pSong = GetSongPtr(nRow);
	pSong->SetPlaying(bOn);
	
}

//------------------------------------------------------------------------------

void
ActiveList::clear()
{
	freeze();
	ClearData();
	Gtk::CList::clear();
	thaw();
}

//------------------------------------------------------------------------------

void
ActiveList::GetSong(int nRow, Song& song) const
{
	Song* pSong = GetSongPtr(nRow);
	song = *pSong;
}

//------------------------------------------------------------------------------

bool
ActiveList::IsSelected(int nRow) const
{
	return (GetSongPtr(nRow))->IsSelected();
}

//------------------------------------------------------------------------------

void
ActiveList::GetAllSongs(SongList& listSongs) const
{
	listSongs.clear();
	
	Gtk::CList_Helpers::RowList::const_iterator itr;
	for (itr = rows().begin(); itr != rows().end(); ++itr) {
		Song* pSong = (Song *)(itr->get_data());
		listSongs.push_back(*pSong);
	}
}

//------------------------------------------------------------------------------

void
ActiveList::GetSelectedSongs(SongList& listSongs) const
{
	listSongs.clear();
	
	Gtk::CList_Helpers::RowList::const_iterator itr;
	for (itr = rows().begin(); itr != rows().end(); ++itr) {
		Song* pSong = (Song *)(itr->get_data());
		if (pSong->IsSelected()) {
			listSongs.push_back(*pSong);
		}
	}
}

//------------------------------------------------------------------------------

int
ActiveList::GetSelectionListIndex(unsigned int nNum) const
{
	Assert(nNum >= 0, "nNum < 0");
	Assert(nNum < rows().size(), "nNum > list size");

	unsigned int nRow = 0;
	int nIndex = 0;

	Gtk::CList_Helpers::RowList::const_iterator itr;
	for (itr = rows().begin(); itr != rows().end(); ++itr) {
		Song* pSong = (Song *)(itr->get_data());
		if (pSong->IsSelected()) {
			if (nRow == nNum) {
				return nIndex;
			}
			nIndex++;
		}
		nRow++;
	}

	return -1;
}

//------------------------------------------------------------------------------

void
ActiveList::DNDDataGet(GdkDragContext* pContext,
					   GtkSelectionData* pSelectionData,
					   guint info,
					   guint time)
{
	EM::I()->Trace() << FILELINE << "ActiveList DNDDataGet" << endl;
	if (info == InterfaceTypes::TARGET_STRING) {
		EM::I()->Trace() << "\tTARGET_STRING" << endl;

		Gtk::CList_Helpers::SelectionList sslist = selection();

		std::string strData;
		for	(Gtk::CList_Helpers::SelectionList::const_iterator itr =
				 sslist.begin();
			 itr != sslist.end();
			 ++itr) {
			EM::I()->Noise() << "row: " << itr->get_row_num() << endl;
			Song* pSong = (Song*)(itr->get_data());
			strData += pSong->GetFilename() + "*";
			EM::I()->Trace() << "filename: " << pSong->GetFilename() << endl;
		}

		EM::I()->Noise() << "full data: " << strData << endl;
		
		gtk_selection_data_set(pSelectionData,
							   pSelectionData->target,
							   8, (guchar*) strData.c_str(), strData.length());
	}
	else if (info == InterfaceTypes::TARGET_ROWNUMS) {
		EM::I()->Trace() << "\tTARGET_ROWNUMS" << endl;

	}
}

//------------------------------------------------------------------------------

void
ActiveList::DNDReceive(GdkDragContext* pContext, gint x, gint y,
					   GtkSelectionData* pData,
					   guint info,
					   guint32 time)
{
	EM::I()->Trace() << "DNDReceive info = " << info << " length = " <<
		pData->length << " format = " << pData->format << " x = " <<
		x << " y = " << y << endl;

	int nRow, nCol;
	get_selection_info(x, y, &nRow, &nCol);
	EM::I()->Trace() << "row = " << nRow << ", col = " << nCol << endl;

	if (drag_get_source_widget(pContext) == this) {
		EM::I()->Trace() << "dragged onto self" << endl;
		if (m_bReorderable) {
			EM::I()->Trace() << "reorder allowed" << endl;
			
			std::string strData((char*)(pData->data));
			EM::I()->Trace() << "DNDReceive data: " << strData << endl;
			
		}
	}
	else {
		EM::I()->Trace() << "drag from different widget" << endl;
	}
}

//------------------------------------------------------------------------------

void
ActiveList::DNDDelete(GdkDragContext* pContext)
{
	// gets here when list is reordered
	EM::I()->Trace() << "DNDDelete" << endl;
	Reordered();
	UpdateBackgroundColors();
}

//------------------------------------------------------------------------------

void
ActiveList::SetReorderable(bool bIn)
{
	set_reorderable(bIn);
}

//------------------------------------------------------------------------------

Song*
ActiveList::GetSongPtr(unsigned int nRow)
{
	Assert(nRow >= 0, "nRow < 0");
	Assert(nRow < rows().size(), "nRow >= rows.size()");
	
	return (Song*)(row(nRow).get_data());
}

//------------------------------------------------------------------------------

Song*
ActiveList::GetSongPtr(unsigned int nRow) const
{
	Assert(nRow >= 0, "nRow < 0");
	Assert(nRow < rows().size(), "nRow >= rows.size()");
	
	return (Song*)(row(nRow).get_data());
}

//------------------------------------------------------------------------------

void
ActiveList::ColumnClick(int nColumn)
{
	EM::I()->Trace() << "column:  " << nColumn << endl;
	if (nColumn != 0) {
		freeze();
		
		set_sort_column(nColumn);
		
		// if this is already the sort column, switch the sort order
		if ((unsigned int)nColumn == m_nSortColumn) {
			if (m_nSortType == GTK_SORT_ASCENDING) {
				m_nSortType = GTK_SORT_DESCENDING;
			}
			else {
				m_nSortType = GTK_SORT_ASCENDING;
			}
		}
		else {
			m_nSortType = GTK_SORT_ASCENDING;
		}
		set_sort_type(m_nSortType);
		
		sort();
		UpdateBackgroundColors();
		
		thaw();

		m_nSortColumn = nColumn;
	}
}

//------------------------------------------------------------------------------

void
ActiveList::UpdateBackgroundColors()
{
	freeze();
	Gtk::CList_Helpers::RowList::const_iterator itr;
	for (itr = rows().begin(); itr != rows().end(); ++itr) {
		if (itr->get_row_num() % 2 == 1) {
			itr->set_background(m_ColorOdd);
		}
		else {
			itr->set_background(m_ColorEven);
		}
	}
	thaw();
}

//------------------------------------------------------------------------------

bool
ActiveList::SetSongPlaying(const Song& song)
{
	m_SongPlaying = song;
	m_bSongPlaying = true;

	return UpdateSongPlaying();
}

//------------------------------------------------------------------------------

bool
ActiveList::UpdateSongPlaying()
{
	bool bFound = false;

	freeze();
	
	if (m_bSongPlaying) {

		Gtk::CList_Helpers::RowList::const_iterator itr;
		for (itr = rows().begin(); itr != rows().end(); ++itr) {
			Song* pSong = (Song*)itr->get_data();
			if (pSong != NULL && *pSong == m_SongPlaying) {
				bFound = true;
				SetAudio(itr->get_row_num(), true);
			}
			else {
				SetAudio(itr->get_row_num(), false);
			}
		}
	}

	thaw();

	return bFound;
}

//------------------------------------------------------------------------------

void
ActiveList::SetNoSongPlaying()
{
	m_bSongPlaying = false;

	freeze();
	
	Gtk::CList_Helpers::RowList::const_iterator itr;
	for (itr = rows().begin(); itr != rows().end(); ++itr) {
		SetAudio(itr->get_row_num(), false);
	}

	thaw();
}

//------------------------------------------------------------------------------

void
ActiveList::ClearData()
{
	Gtk::CList_Helpers::RowList::iterator itr;
	for (itr = rows().begin(); itr != rows().end(); ++itr) {
		Song* pSong = (Song *)(itr->get_data());
		delete pSong;
		itr->set_data(NULL);
	}
}	

//------------------------------------------------------------------------------

void
ActiveList::CheckAll()
{
	Gtk::CList_Helpers::RowList::iterator itr;
	for (itr = rows().begin(); itr != rows().end(); ++itr) {
		
		(*itr)[0].set_pixmap(m_PMCboxNoAudioFull.GetPixmap(),
							 m_PMCboxNoAudioFull.GetMask());
		
		Song* pSong = (Song *)(itr->get_data());
		pSong->SetSelected(true);
	}

}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
