// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// AudioCD.h
//
// $Id: AudioCD.h,v 1.14 2002/02/14 20:27:25 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __AudioCD__
#define __AudioCD__

extern "C" {
#include <cdaudio.h>
}

#include <memory>
#include <sigc++/object.h>
#include <sigc++/slot.h>

#include "SongList.h"
#include "Source.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class AudioCD : public Source, public SigC::Object
{
	// Constants
	private:
		static const int s_nInvalidDevice = -1;
	// Types
	// Variables
	private:
		int m_nDevice;
		auto_ptr<disc_data> m_pDiscData;
		bool m_bDataCurrent;

		SigC::Connection m_Connection;
		struct cddb_query m_Query;
		struct cddb_entry m_Entry;
		int m_nSocket;
		
	// Methods
	public:
		AudioCD();
		~AudioCD();

		void Update();
		
		virtual const SongList& GetSongList();
		
		void Dump() const;

		void Play(int nTrack);
		void Stop();
		void Eject();

		bool CheckCapability(std::string& strMessage);

		bool MediaChanged();

		std::string GetArtist() const;
		std::string GetTitle() const;

		bool IsAudio() const;
		
	private:
		void GetCDDBData();
		void CheckCDDBMessage() const;
		bool CheckDiscInDrive() const;

		void ChoiceDialogDone(int n);
		
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
