// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ChoiceDialog.cpp
//
// $Id: ChoiceDialog.cpp,v 1.7 2002/02/18 08:55:58 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "ChoiceDialog.h"

#include <sigc++/object.h>
#include <sigc++/signal_system.h>
#include <xb/ErrorManager.h>

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

ChoiceDialog* ChoiceDialog::m_pThisInstance(NULL);

//------------------------------------------------------------------------------

ChoiceDialog::ChoiceDialog() :
	xb::XDialog()
{
	m_pLabel = manage(new Gtk::Label());
	get_vbox()->pack_start(*m_pLabel);

	m_pButton = manage(new Gtk::Button("OK"));
	get_action_area()->pack_start(*m_pButton);

	m_pButton->clicked.connect(SigC::slot(this, &ChoiceDialog::hide_all));
	m_pButton->clicked.connect(SigC::slot(this, &ChoiceDialog::SendSignal));

	/*
	// radio buttons
	Gtk::RadioButton* pR = manage(new Gtk::RadioButton(m_Group, "button 1"));
	m_Group = pR->group();
	get_vbox()->pack_start(*pR);
	pR = manage(new Gtk::RadioButton(m_Group, "button 2"));
	get_vbox()->pack_start(*pR);
	pR = manage(new Gtk::RadioButton(m_Group, "button 3"));
	get_vbox()->pack_start(*pR);
	pR = manage(new Gtk::RadioButton(m_Group, "button 4"));
	get_vbox()->pack_start(*pR);
	pR = manage(new Gtk::RadioButton(m_Group, "button 5"));
	get_vbox()->pack_start(*pR);
	*/

	AddChoice("button 1");
	AddChoice("button 2");
	AddChoice("button 3");
	AddChoice("button 4");
	AddChoice("button 5");
	
	show_all();
	
	set_modal(true);
}

//------------------------------------------------------------------------------

ChoiceDialog*
ChoiceDialog::Instance()
{
	if (m_pThisInstance == NULL) {
		m_pThisInstance = new ChoiceDialog;
	}

	return m_pThisInstance;
}

//------------------------------------------------------------------------------

void
ChoiceDialog::ClearChoices()
{
	list<Gtk::RadioButton*>::const_iterator itr;
	for (itr = m_listWidgets.begin(); itr != m_listWidgets.end(); ++itr) {
		get_vbox()->remove(**itr);
	}

	m_listWidgets.clear();
}

//------------------------------------------------------------------------------

void
ChoiceDialog::AddChoice(const std::string& strText)
{
	Gtk::RadioButton* pR = manage(new Gtk::RadioButton(m_Group, strText));
	if (m_listWidgets.size() == 0) {
		m_Group = pR->group();
	}
	
	get_vbox()->pack_start(*pR);
	m_listWidgets.push_back(pR);

	show_all();
}

//------------------------------------------------------------------------------

void
ChoiceDialog::SetText(const std::string& strText)
{
	AssertNotNull(m_pLabel);
	
	m_pLabel->set_text(strText);
}

//------------------------------------------------------------------------------

void
ChoiceDialog::SendSignal()
{
	int nResult = 0;

	int nCount = 0;
	list<Gtk::RadioButton*>::const_iterator itr;
	for (itr = m_listWidgets.begin(); itr != m_listWidgets.end(); ++itr) {
		if ((*itr)->get_active()) {
			nResult = nCount;
		}
		nCount++;
	}
	
	ChoiceDialogDone(nResult);
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
