// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ConfigDialog.cpp
//
// $Id: ConfigDialog.cpp,v 1.22 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "ConfigDialog.h"

#include <config.h>

#include <gtk--/button.h>
#include <gtk--/buttonbox.h>
#include <gtk--/frame.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/table.h>
#include <xb/ErrorManager.h>
#include <xb/String.h>

#include "Controller.h"
#include "Library.h"
#include "PropList.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

ConfigDialog* ConfigDialog::m_pThisInstance(NULL);

//------------------------------------------------------------------------------

ConfigDialog::ConfigDialog() :
	xb::XDialog(),
	m_adjEncoderQuality(50.0, 0.0, 100.0),
	m_scaleEncoderQuality(m_adjEncoderQuality)
{
	set_usize(485, 350);
	set_name("config_dialog");
	
	// widget setup
	get_vbox()->pack_start(m_Notebook);

	Gtk::HButtonBox* pButtonBox = manage(new Gtk::HButtonBox());
	Gtk::Button* pCancel = manage(new Gtk::Button("Cancel"));
	pCancel->set_name("buttonnormal1");
	Gtk::Button* pOK = manage(new Gtk::Button("OK"));
	pButtonBox->pack_start(*pCancel);
	pButtonBox->pack_start(*pOK);

	get_action_area()->pack_start(*pButtonBox);

	// notebook setup
	m_Notebook.set_tab_pos(GTK_POS_TOP);

	Gtk::Frame* pFrame = manage(new Gtk::Frame("About"));
	pFrame->set_border_width(10);
	pFrame->show();

	Gtk::Text* pText = manage(new Gtk::Text());
	Gtk::ScrolledWindow* pSW = manage(new Gtk::ScrolledWindow());
	pSW->set_policy( GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	pSW->add(*pText);
	pFrame->add(*pSW);
	SetAboutText(pText);
	
	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pFrame, "About"));

	pFrame = manage(new Gtk::Frame("Burn"));
	pFrame->set_border_width(10);
	pFrame->set_usize(150, 150);
	pFrame->show();

	Gtk::VBox* pBox = manage(new Gtk::VBox());
	pFrame->add(*pBox);

	Gtk::HBox* pHBox = manage(new Gtk::HBox());
	Gtk::Label* pLabel = manage(new Gtk::Label("Device:"));
	pHBox->pack_start(*pLabel);
	pHBox->pack_start(m_EntryBurnDevice);
	pBox->pack_start(*pHBox);
	
	pHBox = manage(new Gtk::HBox());
	pLabel = manage(new Gtk::Label("Burn Speed:"));
	pHBox->pack_start(*pLabel);
	pHBox->pack_start(m_EntryBurnSpeed);
	pBox->pack_start(*pHBox);

	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pFrame, "Burn"));

	// rip
	// 
	pFrame = manage(new Gtk::Frame("Rip"));
	pFrame->set_border_width(10);
	pFrame->set_usize(150, 150);
	pFrame->show();

	pBox = manage(new Gtk::VBox());
	pFrame->add(*pBox);

	Gtk::Table* pTable = manage(new Gtk::Table(5, 2));
	pBox->pack_start(*pTable);

	pLabel = manage(new Gtk::Label("Device:"));
	pTable->attach(*pLabel, 0, 1, 0, 1);
	pTable->attach(m_EntryRipDevice, 1, 2, 0, 1);
	
	pLabel = manage(new Gtk::Label("Rip Speed:"));
	pTable->attach(*pLabel, 0, 1, 1, 2);
	pTable->attach(m_EntryRipSpeed, 1, 2, 1, 2);
	
	// encoder
	//
	pLabel = manage(new Gtk::Label("Encoder:"));

	pTable->attach(*pLabel, 0, 1, 2, 3);
	
	Gtk::VBox* pBox2 = manage(new Gtk::VBox());
	pTable->attach(*pBox2, 1, 2, 2, 3);

	m_PluginRadioList.clear();
	Gtk::RadioButton_Helpers::Group group;
	Controller::PluginList listPlugins;
	Controller::GetFileOutputPluginList(listPlugins);
	Controller::PluginList::const_iterator itr;
	for (itr = listPlugins.begin(); itr != listPlugins.end(); ++itr) {
		EM::I()->Trace() << "plugin: " << itr->m_strLibName << " -- " <<
			itr->m_strDescription << endl;
		Gtk::RadioButton* pR =
			manage(new Gtk::RadioButton(group, itr->m_strDescription, 0.0));
		group = pR->group();

		pBox2->pack_start(*pR);
		if (System::GetPropList()->GetValue("Encoder") == itr->m_strLibName) {
			pR->set_active(true);
		}

		PluginRadio pradio;
		pradio.m_pRadio = pR;
		pradio.m_strLibName = itr->m_strLibName;

		m_PluginRadioList.push_back(pradio);
	}
	
	pLabel = manage(new Gtk::Label("Encoder Quality:"));
	pTable->attach(*pLabel, 0, 1, 3, 4);
	Gtk::HBox* pQBox = manage(new Gtk::HBox());
	pTable->attach(*pQBox, 1, 2, 3, 4);

	pLabel = manage(new Gtk::Label("L"));
	pQBox->pack_start(*pLabel, false, false, 5);
	pQBox->pack_start(m_scaleEncoderQuality);
	pLabel = manage(new Gtk::Label("H"));
	pQBox->pack_start(*pLabel, false, false, 5);
	
	pLabel = manage(new Gtk::Label("Encoder Bitrate:"));
	pTable->attach(*pLabel, 0, 1, 4, 5);
	pTable->attach(m_EntryBitrate, 1, 2, 4, 5);
	
	Gtk::Frame* pFrame2 = manage(new Gtk::Frame("CD Playlist Naming"));
	pBox->pack_start(*pFrame2);
	pBox2 = manage(new Gtk::VBox());
	pFrame2->add(*pBox2);
	m_pRadioPlaylistCaps = manage(new Gtk::RadioButton("ARTIST - Album Name"));
	m_pRadioPlaylistNormal = manage(new Gtk::RadioButton("Artist - Album Name"));
	m_pRadioPlaylistNormal->set_group(m_pRadioPlaylistCaps->group());
	pBox2->pack_start(*m_pRadioPlaylistCaps);
	pBox2->pack_start(*m_pRadioPlaylistNormal);
	
	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pFrame, "Rip"));
	
	pFrame = manage(new Gtk::Frame("Interface"));
	pFrame->set_border_width(10);
	pFrame->set_usize(150, 150);
	pFrame->show();
	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pFrame, "Interface"));

	pFrame = manage(new Gtk::Frame("Tests"));
	pFrame->set_border_width(10);
	pFrame->set_usize(150, 150);
	pFrame->show();

	pBox = manage(new Gtk::VBox());
	pFrame->add(*pBox);

	Gtk::Button* pButton = manage(new Gtk::Button("Player Stress Test (same song)"));
	pBox->pack_start(*pButton, false, false);
	pButton->clicked.connect(PlayerStressTest.slot());

	pButton = manage(new Gtk::Button("Player Stress Test (sequential)"));
	pBox->pack_start(*pButton, false, false);
	pButton->clicked.connect(PlayerStressTestSeq.slot());

	pButton = manage(new Gtk::Button("Burn Stress Test"));
	pBox->pack_start(*pButton, false, false);
	pButton->clicked.connect(BurnStressTest.slot());

	pButton = manage(new Gtk::Button("Rip Stress Test"));
	pBox->pack_start(*pButton, false, false);
	pButton->clicked.connect(RipStressTest.slot());

	pButton = manage(new Gtk::Button("Recalculate Song Times"));
	pBox->pack_start(*pButton, false, false);
	pButton->clicked.connect(SigC::slot(System::GetLibrary(), &Library::ResetTimes));

	pButton = manage(new Gtk::Button("Clear Library Cache"));
	pBox->pack_start(*pButton, false, false);
	pButton->clicked.connect(SigC::slot(System::GetLibrary(), &Library::ClearCache));
	
	// pButton->set_name("buttontest");
	
	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pFrame, "Tests"));
	
	// signals
	pOK->clicked.connect(SigC::slot(this, &ConfigDialog::hide_all));
	pOK->clicked.connect(SigC::slot(this, &ConfigDialog::ButtonOK));
	pCancel->clicked.connect(SigC::slot(this, &ConfigDialog::hide_all));

	set_modal(true);

	// initial values
	m_EntryBurnDevice.set_text(System::GetPropList()->GetValue("CDRecordDevice"));
	m_EntryBurnSpeed.set_text(System::GetPropList()->GetValue("BurnSpeed"));
	m_EntryRipDevice.set_text(System::GetPropList()->GetValue("CDReader"));
	m_EntryRipSpeed.set_text(System::GetPropList()->GetValue("RipSpeed"));
	if (System::GetPropList()->GetValue("CDPlaylistCaps") == "1") {
		m_pRadioPlaylistCaps->set_active(true);
	}
	else {
		m_pRadioPlaylistNormal->set_active(true);
	}
	m_EntryBitrate.set_text(System::GetPropList()->GetValue("EncoderBitRate"));
	m_adjEncoderQuality.set_value(
		System::GetPropList()->GetIntValue("EncoderQuality"));
}

//------------------------------------------------------------------------------

ConfigDialog*
ConfigDialog::Instance()
{
	if (m_pThisInstance == NULL) {
		m_pThisInstance = new ConfigDialog;
	}

	return m_pThisInstance;
}

//------------------------------------------------------------------------------

void
ConfigDialog::SetAboutText(Gtk::Text* pText)
{
	pText->set_word_wrap(true);

	pText->insert(std::string("xtunes v") + VERSION + "\n");
	pText->insert("Copyright 2001 (C) Patrick Crosby <pcrosby@pobox.com>\n\n");
	pText->insert("Licensed under the GNU General Public License v2\n\n");
	pText->insert("Credits:\n\n");
	pText->insert("Programming:  Patrick Crosby <pcrosby@pobox.com>\n\n");
	pText->insert("Artwork:  Geoff Koops <geoffkoops@hotmail.com> and Patrick Crosby\n\n");
	pText->insert("Uses the following libraries/applications:\n\n");
	pText->insert("gtkmm:  Copyright (C) 1997 Elliot Lee <sopwith@redhat.com>, Phil Dawes, Tero Pulkkinen, Guillaume Laurent\n\n");
	pText->insert("lame:  Copyright (C) 1999 Mark Taylor\n\n");
	// pText->insert("mad:   Copyright (C) 2000-2001 Robert Leslie\n\n");
	pText->insert("cdparanoia:  Copyright (C) 2001 Xiph.org and Heiko Eissfeldt <heiko@escape.colossus.de>\n\n");
	pText->insert("cdaudio:  Copyright (C) 1998-99 Tony Arcieri\n\n");
	pText->insert("id3lib:   Copyright 1999, 2000  Scott Thomas Haug\n\n");
	pText->insert("libPropList:  Copyright (C) 1997 Bjoern Giesler <giesler@ira.uka.de>\n\n");
	pText->insert("cdrecord:  Copyright (C) 1995-2001 Jrg Schilling\n\n");
	pText->insert("mpg123:  Copyright (C) 1995-99 by Michael Hipp\n\n");
	pText->insert("gdkxft:  Copyright (C) 2001 Josh Parsons\n\n");
}

//------------------------------------------------------------------------------

void
ConfigDialog::ButtonOK()
{
	System::GetPropList()->SetValue("CDRecordDevice",
									m_EntryBurnDevice.get_text());
	System::GetPropList()->SetValue("BurnSpeed",
									m_EntryBurnSpeed.get_text());
	System::GetPropList()->SetValue("CDReader",
									m_EntryRipDevice.get_text());
	System::GetPropList()->SetValue("RipSpeed",
									m_EntryRipSpeed.get_text());
	std::string s("0");
	if (m_pRadioPlaylistCaps->get_active()) {
		s = "1";
	}
	System::GetPropList()->SetValue("CDPlaylistCaps", s);

	bool bFound = false;
	PluginRadioList::const_iterator itr;
	for (itr = m_PluginRadioList.begin();
		 itr != m_PluginRadioList.end() && !bFound;
		 ++itr) {

		if (itr->m_pRadio && itr->m_pRadio->get_active()) {
			System::GetPropList()->SetValue("Encoder", itr->m_strLibName);
			bFound = true;
		}
	}
	if (!bFound) {
		EM::I()->Warn() <<
			"none of the encoder radio buttons active...how can this be?" <<
			endl;
	}
	
	System::GetPropList()->SetValue("EncoderQuality",
									xb::String((int)m_adjEncoderQuality.get_value()));
	System::GetPropList()->SetValue("EncoderBitRate",
									m_EntryBitrate.get_text());

	System::GetPropList()->Save();
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
