// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// InfoDialog.cpp
//
// $Id: InfoDialog.cpp,v 1.11 2002/03/11 23:35:06 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "InfoDialog.h"

#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/buttonbox.h>
#include <gtk--/frame.h>
#include <list>
#include <xb/ErrorManager.h>
#include <xb/FileUtil.h>

#include "Controller.h"
#include "Info.h"
#include "Library.h"
#include "System.h"
#include "Util.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

InfoDialog* InfoDialog::m_pThisInstance(NULL);

//------------------------------------------------------------------------------

InfoDialog::InfoDialog() :
	xb::XDialog(),
	m_clistGeneral(2),
	m_clistDetail(2)
{
	get_vbox()->pack_start(m_entryTitle);
	
	get_vbox()->pack_start(m_Notebook);

	Gtk::HButtonBox* pButtonBox = manage(new Gtk::HButtonBox());
	Gtk::Button* pPrev = manage(new Gtk::Button("Prev Song"));
	Gtk::Button* pNext = manage(new Gtk::Button("Next Song"));
	pButtonBox->pack_start(*pPrev, false, false);
	pButtonBox->pack_start(*pNext, false, false);
	Gtk::Button* pCancel = manage(new Gtk::Button("Cancel"));
	Gtk::Button* pOK = manage(new Gtk::Button("OK"));
	pButtonBox->pack_end(*pCancel, false, false);
	pButtonBox->pack_end(*pOK, false, false);
	get_action_area()->pack_start(*pButtonBox);

	m_Notebook.set_tab_pos(GTK_POS_TOP);

	// info tab
	Gtk::Box* pBox = manage(new Gtk::HBox());
	Gtk::Frame* pFrame1 = manage(new Gtk::Frame("General"));
	pBox->pack_start(*pFrame1);
	Gtk::Frame* pFrame2 = manage(new Gtk::Frame("Detail"));
	pBox->pack_start(*pFrame2);

	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pBox, "Info"));

	// info::general frame
	pFrame1->add(m_clistGeneral);

	std::list<std::string> listColumns;
	listColumns.push_back(" ");
	listColumns.push_back(" ");
	for (int i = 0; i < 5; i++) {
		m_clistGeneral.rows().push_back(listColumns);
		m_clistDetail.rows().push_back(listColumns);
	}
	
	m_clistGeneral.cell(0, 0).set_text("Type:");
	m_clistGeneral.cell(1, 0).set_text("Time:");
	m_clistGeneral.cell(2, 0).set_text("Size:");
	m_clistGeneral.cell(3, 0).set_text("Date:");
	m_clistGeneral.cell(4, 0).set_text("Filename:");

	m_clistGeneral.column(0).set_resizable(false);
	m_clistGeneral.column(1).set_resizable(false);
	m_clistGeneral.column(0).set_auto_resize(true);
	m_clistGeneral.column(1).set_auto_resize(true);
	m_clistGeneral.column(0).set_justification(GTK_JUSTIFY_RIGHT);
	m_clistGeneral.column(1).set_justification(GTK_JUSTIFY_LEFT);
		
	// info::detail frame
	pFrame2->add(m_clistDetail);
	m_clistDetail.cell(0, 0).set_text("Type:");
	m_clistDetail.cell(1, 0).set_text("Bit Rate:");
	m_clistDetail.cell(2, 0).set_text("Sample Rate:");
	m_clistDetail.cell(3, 0).set_text("Channels:");
	m_clistDetail.cell(4, 0).set_text("Encoded By:");
	
	m_clistDetail.column(0).set_resizable(false);
	m_clistDetail.column(1).set_resizable(false);
	m_clistDetail.column(0).set_auto_resize(true);
	m_clistDetail.column(1).set_auto_resize(true);
	m_clistDetail.column(0).set_justification(GTK_JUSTIFY_RIGHT);
	m_clistDetail.column(1).set_justification(GTK_JUSTIFY_LEFT);

	// tags tab
	Gtk::VBox* pVBox = manage(new Gtk::VBox());

	Gtk::HBox* pHBox = manage(new Gtk::HBox());
	Gtk::Label* pLabel = manage(new Gtk::Label("Artist:"));
	pHBox->pack_start(*pLabel);
	pHBox->pack_start(m_entryTagArtist);
	pVBox->pack_start(*pHBox);

	pHBox = manage(new Gtk::HBox());
	pLabel = manage(new Gtk::Label("Album:"));
	pHBox->pack_start(*pLabel);
	pHBox->pack_start(m_entryTagAlbum);
	pVBox->pack_start(*pHBox);

	pHBox = manage(new Gtk::HBox());
	pLabel = manage(new Gtk::Label("Track:"));
	pHBox->pack_start(*pLabel);
	pHBox->pack_start(m_entryTagTrack);
	pVBox->pack_start(*pHBox);

	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pVBox, "Tags"));

	// options tab
	pVBox = manage(new Gtk::VBox());

	m_Notebook.pages().push_back(
		Gtk::Notebook_Helpers::TabElem(*pVBox, "Options"));

	// signals
	pCancel->clicked.connect(SigC::slot(this, &InfoDialog::hide_all));
	pOK->clicked.connect(SigC::slot(this, &InfoDialog::ButtonOK));
}

//------------------------------------------------------------------------------

InfoDialog*
InfoDialog::Instance()
{
	if (m_pThisInstance == NULL) {
		m_pThisInstance = new InfoDialog;
	}

	return m_pThisInstance;
}

//------------------------------------------------------------------------------

void
InfoDialog::SetSong(const Song& song, SourceTable::SourceType eType)
{
	m_Song = song;
	m_eSourceType = eType;
	UpdateInfo();
}

//------------------------------------------------------------------------------

void
InfoDialog::UpdateInfo()
{
	Info* pInfo = System::GetController()->GetInfoPluginForFile(
		m_Song.GetFilename());

	EM::I()->Trace() << FILELINE << "UpdateInfo:" << endl;
	pInfo->Dump();
	
	m_entryTitle.set_text(pInfo->GetTitle());

	if (m_eSourceType == SourceTable::LIBRARY ||
		m_eSourceType == SourceTable::PLAYLIST) {
		m_clistGeneral.cell(0, 1).set_text("MP3");
	}
	else if (m_eSourceType == SourceTable::CD) {
		m_clistGeneral.cell(0, 1).set_text("Audio CD");
	}
	else {
		m_clistGeneral.cell(0, 1).set_text("(unknown)");
	}

	m_clistGeneral.cell(1, 1).set_text(Util::GetTimeStr(pInfo->GetSeconds()));
	m_clistGeneral.cell(2, 1).set_text(
		Util::GetFileSizeStr(xb::FileUtil::GetSize(m_Song.GetFilename())));
			
	// m_labelGeneralFilename.set(m_Song.GetFilename());

	
	m_entryTagArtist.set_text(pInfo->GetArtist());
	m_entryTagAlbum.set_text(pInfo->GetAlbum());
	m_entryTagTrack.set_text(pInfo->GetTrack());
}

//------------------------------------------------------------------------------

void
InfoDialog::ButtonOK()
{
	m_Song.SetTitle(m_entryTitle.get_text());
	m_Song.SetArtist(m_entryTagArtist.get_text());
	m_Song.SetAlbum(m_entryTagAlbum.get_text());
	m_Song.SetTrack(m_entryTagTrack.get_text());
	
	Info* pInfo = System::GetController()->GetInfoPluginForFile(
		m_Song.GetFilename());
	pInfo->SetTitle(m_entryTitle.get_text());
	pInfo->SetArtist(m_entryTagArtist.get_text());
	pInfo->SetAlbum(m_entryTagAlbum.get_text());
	pInfo->SetTrack(m_entryTagTrack.get_text());
	pInfo->Save();

	System::GetLibrary()->SetCacheUpdate(true);

	hide_all();
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
