// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Interface.cpp
//
// $Id: Interface.cpp,v 1.94 2002/03/09 19:14:08 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include "Interface.h"

// #include <list>

#include <gdk/gdkkeysyms.h>
#include <gtk--/box.h>
#include <gtk--/frame.h>
#include <gtk--/list.h>
#include <gtk--/paned.h>
#include <gtk--/scrolledwindow.h>
#include <gtk--/window.h>
#include <xb/AutoLock.h>
#include <xb/AutoLockGtk.h>
#include <xb/ButtonImg.h>
#include <xb/ErrorManager.h>
#include <xb/FileUtil.h>
#include <xb/String.h>

#include "ActiveList.h"
#include "ConfigDialog.h"
#include "ConfirmDialog.h"
#include "Controller.h"
#include "Info.h"
#include "InfoDialog.h"
#include "InterfaceTop.h"
#include "Library.h"
#include "MessageWindow.h"
#include "PropList.h"
#include "SourceTable.h"
#include "System.h"
#include "TitleBar.h"
#include "Util.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

pthread_mutex_t Interface::s_ThreadLock = PTHREAD_MUTEX_INITIALIZER;

//------------------------------------------------------------------------------

Interface::Interface() :
	Gtk::Window(GTK_WINDOW_TOPLEVEL),
		 m_bnRandom(INTERFACE_DATA_DIR "/random-up.xpm",
					INTERFACE_DATA_DIR "/random-over.xpm",
					INTERFACE_DATA_DIR "/random-down.xpm"),
		 m_bnLoop(INTERFACE_DATA_DIR "/loop-up.xpm",
				  INTERFACE_DATA_DIR "/loop-over.xpm",
				  INTERFACE_DATA_DIR "/loop-down.xpm"),
		 m_pSourceTable(NULL),
		 m_nRowSelected(-1),
		 m_nPlayPauseState(STATE_STOP),
		 m_nActionState(STATE_BROWSE),
		 m_nCurrentAction(STATE_BROWSE),
		 m_nCurrentFileIndex(0),
		 m_nSelectedSourceType(SourceTable::LIBRARY),
		 m_nSourcePlaying(SourceTable::NONE),
		 m_bShrunk(false),
		 m_nMotionX(0),
		 m_nMotionY(0),
		 m_nPosX(0),
		 m_nPosY(0),
		 m_bDecorate(false)
{
	// general window setup
	set_title("xtunes");
	set_border_width(0);
	set_usize(System::GetPropList()->GetIntValue("AppWidth"),
			  System::GetPropList()->GetIntValue("AppHeight"));


	if (System::GetPropList()->GetIntValue("DecorateWindow") != 0) {
		m_bDecorate = true;
	}
	
	// ---------------------------------------------------------
	// boxes
	// ---------------------------------------------------------

	m_pBoxMaster = manage(new Gtk::VBox(false, 0));
	m_pBoxMaster->set_spacing(10);
	add(*m_pBoxMaster);

	TitleBar* pTitleBar = manage(new TitleBar());
	m_pBoxMaster->pack_start(*pTitleBar, false, false);

	Gtk::VBox* pBoxBorder = manage(new Gtk::VBox());
	pBoxBorder->set_border_width(10);
	pBoxBorder->set_spacing(5);
	m_pBoxMaster->pack_start(*pBoxBorder, true, true);
	
	m_pBoxTop = manage(new InterfaceTop());
	// m_pBoxMaster->pack_start(*m_pBoxTop, false, false);
	pBoxBorder->pack_start(*m_pBoxTop, false, false);

	m_pBoxMiddle = manage(new Gtk::HBox(false, 0));
	m_pBoxMiddle->set_spacing(10);
	// m_pBoxMaster->pack_start(*m_pBoxMiddle);
	pBoxBorder->pack_start(*m_pBoxMiddle);

	m_pBoxBottom = manage(new Gtk::HBox(false, 0));
	m_pBoxBottom->set_spacing(10);
	// m_pBoxMaster->pack_start(*m_pBoxBottom, false, false);
	pBoxBorder->pack_start(*m_pBoxBottom, false, false);
	
	// ---------------------------------------------------------
	// Middle Box Contents
	// ---------------------------------------------------------

	Gtk::Paned* pPane = Gtk::manage(new Gtk::HPaned());
	
	Gtk::VBox* pVBox = Gtk::manage(new Gtk::VBox());
	pVBox->set_spacing(10);
	
	m_pSourceTable = Gtk::manage(new SourceTable());
	m_pSourceTable->set_usize(-1, 190);
	
	Gtk::ScrolledWindow* pSW = manage(new Gtk::ScrolledWindow());
	pSW->set_policy( GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	pSW->add(*m_pSourceTable);

	pVBox->pack_start(*pSW);
	pVBox->pack_start(m_entrySource, false, false);
	
	Gtk::Frame* pFrame = manage(new Gtk::Frame());
	pFrame->set_shadow_type(GTK_SHADOW_IN);
	pFrame->add(*pVBox);
	pPane->pack1(*pFrame, true, true);
	
	m_pActiveList = manage(new ActiveList());
	m_pActiveList->set_row_height(m_pSourceTable->get_row_height());
	
	pSW = manage(new Gtk::ScrolledWindow());
	pSW->set_policy( GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
	pSW->add(*m_pActiveList);

	pFrame = manage(new Gtk::Frame());
	pFrame->set_shadow_type(GTK_SHADOW_IN);
	pFrame->add(*pSW);
	pPane->pack2(*pFrame, true, false);

	pPane->set_gutter_size(20);
	pPane->set_position(System::GetPropList()->GetIntValue("PanePosition"));
	
	
	m_pBoxMiddle->pack_start(*pPane);

	// ---------------------------------------------------------
	// Bottom Box Contents
	// ---------------------------------------------------------
	xb::ButtonImg* pNewPlayList =
		manage(new xb::ButtonImg(INTERFACE_DATA_DIR "/playlist-up.xpm",
								 INTERFACE_DATA_DIR "/playlist-over.xpm",
								 INTERFACE_DATA_DIR "/playlist-down.xpm"));
	xb::ButtonImg* pConfig =
		manage(new xb::ButtonImg(INTERFACE_DATA_DIR "/config-up.xpm",
								 INTERFACE_DATA_DIR "/config-over.xpm",
								 INTERFACE_DATA_DIR "/config-down.xpm"));
	xb::ButtonImg* pEject =
		manage(new xb::ButtonImg(INTERFACE_DATA_DIR "/eject-up.xpm",
								 INTERFACE_DATA_DIR "/eject-over.xpm",
								 INTERFACE_DATA_DIR "/eject-down.xpm"));
	xb::ButtonImg* pVisualization =
		manage(new xb::ButtonImg(INTERFACE_DATA_DIR "/visualization-up.xpm",
								 INTERFACE_DATA_DIR "/visualization-over.xpm",
								 INTERFACE_DATA_DIR "/visualization-down.xpm"));

	m_pBoxBottom->pack_start(*pNewPlayList, false, false);
	m_pBoxBottom->pack_start(m_bnRandom, false, false);
	m_pBoxBottom->pack_start(m_bnLoop, false, false);
	m_pBoxBottom->pack_start(m_labelTime);
	m_pBoxBottom->pack_start(*pConfig, false, false);
	m_pBoxBottom->pack_start(*pVisualization, false, false);
	m_pBoxBottom->pack_start(*pEject, false, false);

	Gtk::ImageLoader loader(INTERFACE_DATA_DIR "/resize.xpm");
	Gtk::Pixmap* pPix =	manage(new Gtk::Pixmap(loader.pix(), loader.bit()));
	// pPix->shape_combine_mask(loader.bit(), 0, 0);
	m_pBoxBottom->pack_start(*pPix, false, false);

	
	// ---------------------------------------------------------
	// signals
	// ---------------------------------------------------------

	// boxtop signals
	m_pBoxTop->ButtonPlayPause.connect(
		SigC::slot(this, &Interface::ButtonPlayPause));
	m_pBoxTop->ButtonPrev.connect(SigC::slot(this, &Interface::ButtonPrev));
	m_pBoxTop->ButtonNext.connect(SigC::slot(this, &Interface::ButtonNext));
	m_pBoxTop->VolumeChanged.connect(
		SigC::slot(this, &Interface::VolumeChanged));
	m_pBoxTop->ButtonAction.connect(SigC::slot(this, &Interface::ButtonAction));
	m_pBoxTop->MessageWinConfirmOK.connect(
		SigC::slot(this, &Interface::MessageWinConfirmOK));
	m_pBoxTop->MessageWinProgressCancel.connect(
		SigC::slot(this, &Interface::ProgressCancel));
	m_pBoxTop->MessageWinProgressClicked.connect(
		SigC::slot(this, &Interface::MessageWinProgressClicked));
	m_pBoxTop->SearchKey.connect(
		SigC::slot(this, &Interface::SearchKey));

	m_pActiveList->select_row.connect(
		SigC::slot(this, &Interface::SongSelected));
	m_pActiveList->Reordered.connect(
		SigC::slot(this, &Interface::ActiveListReordered));
	m_pActiveList->key_press_event.connect(
		SigC::slot(this, &Interface::ActiveListKey));
	m_pActiveList->button_press_event.connect(
		SigC::slot(this, &Interface::ActiveListClick));
	
	m_pSourceTable->select_row.connect(
		SigC::slot(this, &Interface::SourceSelected));
	m_pSourceTable->key_press_event.connect(
		SigC::slot(this, &Interface::SourceTableKey));
	m_entrySource.activate.connect(
		SigC::slot(this, &Interface::SourceEntryActivate));

	pNewPlayList->clicked.connect(
		SigC::slot(this, &Interface::ButtonNewPlayList));
	pEject->clicked.connect(SigC::slot(this, &Interface::ButtonEject));
	pVisualization->clicked.connect(
		SigC::slot(this, &Interface::ButtonVisualization));

	System::GetController()->RipStatusUpdate.connect(
		SigC::slot(this, &Interface::RipStatusUpdate));
	System::GetController()->RipDone.connect(
		SigC::slot(this, &Interface::RipDone));
															 
	System::GetController()->BurnStatusUpdate.connect(
		SigC::slot(this, &Interface::BurnStatusUpdate));
	System::GetController()->BurnDone.connect(
		SigC::slot(this, &Interface::BurnDone));

	System::GetController()->PlayerComplete.connect(
		SigC::slot(this, &Interface::PlayComplete));

	m_ImportDialog.get_ok_button()->clicked.connect(
		SigC::slot(this, &Interface::ButtonImportOK));
	
	pConfig->clicked.connect(SigC::slot(this, &Interface::ButtonConfig));

	if (!m_bDecorate) {
		pTitleBar->button_press_event.connect(
			SigC::slot(this, &Interface::TitlePress));
		pTitleBar->motion_notify_event.connect(
			SigC::slot(this, &Interface::TitleMotion));
	}
	
	pTitleBar->ButtonShrink.connect(SigC::slot(this, &Interface::ButtonShrink));
	
	// done with signals
	
	SetSongList(System::GetLibrary()->GetSongList());
	
	int nVolume = System::GetController()->GetVolume();
	m_pBoxTop->SetVolume(nVolume);

	if (!m_bDecorate) {
		realize_if_needed();
		get_window().set_decorations(GdkWMDecoration(0));
	}
	
	show_all();

	m_pSourceTable->UpdateSources(false);
	m_pSourceTable->UpdateCDInfo();

	m_pBoxTop->SetMessageWinState(MessageWindow::LOGO);
	
	std::string strMessage;
	if (!System::GetController()->CheckCapability(strMessage)) {
		EM::I()->Dialog(strMessage);
	}

	ConfigDialog::Instance()->PlayerStressTest.connect(
		SigC::slot(this, &Interface::PlayerStressTest));
	ConfigDialog::Instance()->PlayerStressTestSeq.connect(
		SigC::slot(this, &Interface::PlayerStressTestSeq));
	ConfigDialog::Instance()->BurnStressTest.connect(
		SigC::slot(this, &Interface::BurnStressTest));
	ConfigDialog::Instance()->RipStressTest.connect(
		SigC::slot(this, &Interface::RipStressTest));
	ConfigDialog::Instance()->hide_all();

	// ChoiceDialog* pDialog = ChoiceDialog::Instance();
	// pDialog->SetText("Inexact CDDB match.  Please choose one:");
	// pDialog->ClearChoices();
	// 
	// for (int i = 0; i < 5; i++) {
	// 	pDialog->AddChoice(xb::String(i) + " choice");
	// }

	// idle function
	Gtk::Main::timeout.connect(slot(this, &Interface::Idle), 10);

}

//------------------------------------------------------------------------------

Interface::~Interface()
{
	if (System::GetPropList()->GetIntValue("SaveInterfaceChangesOnExit")) {
		// save interface values
	}
}

//------------------------------------------------------------------------------

gint
Interface::delete_event_impl(GdkEventAny* pEvent)
{
	Gtk::Main::quit();
	return 0;
}

//------------------------------------------------------------------------------

void
Interface::Init()
{
}

//------------------------------------------------------------------------------

void
Interface::AppendSong(const Song& song)
{
	m_pActiveList->AppendSong(song);
	int nRow = m_pActiveList->rows().size() - 1;
	
	SelectRow(nRow, false);
}

//------------------------------------------------------------------------------

void
Interface::SetSongList(const SongList& listSongs)
{
	m_pActiveList->freeze();
	m_pActiveList->clear();

	int nTotalCount = 0;
	int nActiveCount = 0;

	std::string strSearch = m_pBoxTop->GetSearch();
	SongList::const_iterator itr;
	for (itr = listSongs.begin(); itr != listSongs.end(); ++itr) {
		if ((strSearch.length() == 0) || itr->SatisfiesSearch(strSearch)) {
			AppendSong(*itr);
			nActiveCount++;
		}
		nTotalCount++;
	}

	UpdateSearchLabel(nActiveCount, nTotalCount);
	UpdateTimeLabel();
	
	CheckAllSongs();
	m_pActiveList->UpdateSongPlaying();
	m_pActiveList->thaw();
}

//------------------------------------------------------------------------------

void
Interface::SongSelected(int nRow, int nCol, GdkEvent* pEvent)
{
	EM::I()->Trace() << "SongSelected: nRow == " << nRow << ", nCol == " <<
		nCol << endl;
	if (nCol != 0) {
		m_nRowSelected = nRow;
		if (pEvent && pEvent->type == GDK_2BUTTON_PRESS) {
			m_listSelectedSongs.clear();
			System::GetController()->PlayerStop(false);

			if (!m_pActiveList->IsSelected(nRow)) {
				m_pActiveList->SetCheckbox(nRow, true);
			}

			m_nCurrentFileIndex =
				m_pActiveList->GetSelectionListIndex((unsigned int)nRow);
			
			// random goes off if you double click...
			m_bnRandom.SetState(false);
			PlaySelectedSongs(m_nCurrentFileIndex);
		}
	}
	else {
		// unselect row
		m_pActiveList->row(nRow).unselect();

		if (m_pActiveList->IsSelected(nRow)) {
			SelectRow(nRow, false);
		}
		else {
			SelectRow(nRow, true);
		}
		
		UpdateTimeLabel();
	}
}

//------------------------------------------------------------------------------

void
Interface::SourceSelected(int nRow, int nCol, GdkEvent* pEvent)
{
	std::string strName = m_pSourceTable->GetName(nRow);
	SourceTable::SourceType nType = m_pSourceTable->GetType(nRow);
	m_nSelectedSourceType = nType;
	EM::I()->Trace() << "name:  " << strName << ", type:  " <<
		nType << endl;

	if (pEvent != NULL) {
		// double click to edit playlist names
		EM::I()->Noise() << "event: " << pEvent->type << endl;
		if (pEvent->type == GDK_2BUTTON_PRESS) {
			if (nType == SourceTable::PLAYLIST) {
				m_entrySource.set_text(strName);
				m_entrySource.set_editable(true);
				m_entrySource.grab_focus();
			}
			else {
				m_entrySource.set_text("");
				m_entrySource.set_editable(false);
			}
		}
		else {
			m_entrySource.set_text("");
		}
	}

	if (strName != m_strSelectedSource) {
		m_strSelectedSource = strName;
		UpdateSourceSongList();

		// if paused, stop since the source has changed
		//
		if (m_nPlayPauseState == STATE_PAUSE) {
			System::GetController()->PlayerStop(false);
			SwitchState(STATE_STOP);
		}

		// if they select the cd while burning or ripping, set
		// message window state back to progress
		//
		if (nType == SourceTable::CD) {
			if (System::GetController()->BurnAvailable() == false ||
				System::GetController()->RipAvailable() == false) {

				m_pBoxTop->SetMessageWinState(MessageWindow::PROGRESS);
				
			}
		}
	}


	
}

//------------------------------------------------------------------------------

void
Interface::UpdateSourceSongList()
{
	Source* pSource = NULL;
	
	if (m_nSelectedSourceType == SourceTable::LIBRARY) {
		pSource = System::GetLibrary();
		m_pBoxTop->SetActionType(InterfaceTop::IMPORT);
		m_nActionState = STATE_BROWSE;
		m_pActiveList->SetReorderable(false);
	}
	else if (m_nSelectedSourceType == SourceTable::CD) {
		pSource = System::GetController()->GetAudioCD();
		m_pBoxTop->SetActionType(InterfaceTop::RIP);
		m_nActionState = STATE_RIP;
		m_pActiveList->SetReorderable(false);
	}
	else if (m_nSelectedSourceType == SourceTable::PLAYLIST) {
		pSource = System::GetController()->GetPlayList(m_strSelectedSource);
		m_pBoxTop->SetActionType(InterfaceTop::BURN);
		m_nActionState = STATE_BURN;
		m_pActiveList->SetReorderable(true);
	}
	else {
		EM::I()->Warn() << "unhandled source type:  " <<
			m_nSelectedSourceType << endl;
	}

	if (pSource != NULL) {
		EM::I()->Trace() << "getting song list..." << endl;
		SetSongList(pSource->GetSongList());
		UpdateTimeLabel();
	}
	else {
		EM::I()->Warn() << "no source found..." << endl;
	}
}

//------------------------------------------------------------------------------

void
Interface::SourceEntryActivate()
{
	if (m_entrySource.get_text().length() > 0) {
		EM::I()->Trace() << "rename playlist:  " << m_entrySource.get_text() <<
			endl;
		EM::I()->Trace() << "original playlist: " << m_strSelectedSource << endl;
		System::GetController()->RenamePlayList(m_strSelectedSource,
												m_entrySource.get_text());
		m_pSourceTable->UpdateSources(true);
		m_entrySource.set_text("");
		m_entrySource.set_editable(false);
	}
}

//------------------------------------------------------------------------------

void
Interface::ButtonPlayPause()
{
	if (m_nPlayPauseState == STATE_STOP) {
		PlaySelectedSongs(0);
	}
	else if (m_nPlayPauseState == STATE_PAUSE) {
		if (m_nSourcePlaying == SourceTable::CD) {
			// PATRICK: fix
//			System::GetController()->CDStop();
		}
		else {
			System::GetController()->PlayerPause(false);
			SwitchState(STATE_PLAY);
		}
	}
	else if (m_nPlayPauseState == STATE_PLAY) {
		/*
		m_listSelectedSongs.clear();
		if (m_nSourcePlaying == SourceTable::CD) {
			System::GetController()->CDStop();
		}
		else {
			System::GetController()->PlayerStop(false);
		}
		
		SwitchState(STATE_PAUSE);
		m_pActiveList->SetNoSongPlaying();
		*/
		if (m_nSourcePlaying == SourceTable::CD) {
			System::GetController()->CDStop();
			SwitchState(STATE_STOP);
		}
		else {
			System::GetController()->PlayerPause(true);
			SwitchState(STATE_PAUSE);
		}
		
	}
}

//------------------------------------------------------------------------------

void
Interface::ButtonNext()
{
	if (m_nPlayPauseState == STATE_PLAY) {
		if (m_nSourcePlaying == SourceTable::CD) {
			m_nCurrentFileIndex++;
			if (m_nCurrentFileIndex >= m_listSelectedSongs.size()) {
				m_nCurrentFileIndex = m_listSelectedSongs.size() - 1;
			}
			System::GetController()->CDPlay(
				atoi(m_listSelectedSongs[m_nCurrentFileIndex].GetTrack().c_str()));
			UpdateMessageWindow();
		}
		else {
			System::GetController()->PlayerStop(false);
			PlayComplete();
		}
	}
}

//------------------------------------------------------------------------------

void
Interface::ButtonPrev()
{
	if (m_nPlayPauseState == STATE_PLAY) {
		if (m_nSourcePlaying == SourceTable::CD) {
			long nAdj = m_nCurrentFileIndex - 1;
			if (nAdj < 0) {
				m_nCurrentFileIndex = 0;
			}
			else {
				m_nCurrentFileIndex--;
			}

			// System::GetController()->CDStop();
			System::GetController()->CDPlay(
				atoi(m_listSelectedSongs[m_nCurrentFileIndex].GetTrack().c_str()));
			UpdateMessageWindow();
		}
		else {
			long nAdj = m_nCurrentFileIndex - 1;
			if (nAdj < 0) {
				m_nCurrentFileIndex = 0;
			}
			else {
				m_nCurrentFileIndex--;
			}
			
			System::GetController()->PlayerStop(false);
			PlaySelectedSongs(m_nCurrentFileIndex);
		}
	}
}

//------------------------------------------------------------------------------

void
Interface::PlayComplete()
{
	EM::I()->Trace() << "PlayComplete" << endl;
	m_nCurrentFileIndex++;
	if (m_nCurrentFileIndex < m_listSelectedSongs.size()) {
		EM::I()->Trace() << "PlayComplete: playing next song" << endl;
		System::GetController()->PlayerPlay(m_listSelectedSongs[m_nCurrentFileIndex].GetFilename());
		m_pActiveList->SetSongPlaying(m_listSelectedSongs[m_nCurrentFileIndex]);
		UpdateMessageWindow();
	}
	else {
		if (m_bnLoop.IsToggled() == true) {
			EM::I()->Trace() << "PlayComplete: looping to beginning of list" <<
				endl;
			m_nCurrentFileIndex = 0;
			System::GetController()->PlayerPlay(m_listSelectedSongs[m_nCurrentFileIndex].GetFilename());
			m_pActiveList->SetSongPlaying(m_listSelectedSongs[m_nCurrentFileIndex]);
			UpdateMessageWindow();
		}
		else {
			EM::I()->Trace() << "PlayComplete: done playing list" << endl;
			m_listSelectedSongs.clear();
			m_nCurrentFileIndex = 0;
			SwitchState(STATE_STOP);
			m_pActiveList->SetNoSongPlaying();
			System::GetController()->PlayerStop();
		}
	}

	EM::I()->Trace() << "PlayComplete: done" << endl;
}

//------------------------------------------------------------------------------

void
Interface::PlaySelectedSongs(unsigned int nStartIndex)
{
	m_pActiveList->GetSelectedSongs(m_listSelectedSongs);
	if (m_listSelectedSongs.size() > 0) {

		if (m_bnRandom.IsToggled() == true) {
			m_listSelectedSongs.Randomize();
		}
		
		m_nCurrentFileIndex = nStartIndex;
		Assert(nStartIndex < m_listSelectedSongs.size(),
			   "start index > song list size");

		if (m_nSourcePlaying == SourceTable::NONE) {
			m_nSourcePlaying = m_nSelectedSourceType;
		}
		
		if (m_nSourcePlaying == SourceTable::CD) {

			System::GetController()->CDPlay(
				atoi(m_listSelectedSongs[m_nCurrentFileIndex].GetTrack().c_str()));
			
		}
		else {
			
			Info* pInfo = System::GetController()->GetInfoPluginForFile(
				m_listSelectedSongs[m_nCurrentFileIndex].GetFilename());
			if (pInfo != NULL) {
				pInfo->Dump();
			}
			
			System::GetController()->PlayerPlay(
				m_listSelectedSongs[m_nCurrentFileIndex].GetFilename());
			
		}

		EM::I()->Trace() << "al->setsongplaying, switchstate" << endl;
		
		m_pActiveList->SetSongPlaying(m_listSelectedSongs[m_nCurrentFileIndex]);
		SwitchState(STATE_PLAY);

		EM::I()->Trace() << "al->setsongplaying, switchstate -- done." << endl;
	}
}

//------------------------------------------------------------------------------

void
Interface::SwitchState(int nState)
{
	EM::I()->Trace() << "SwitchState: nState == " << nState << endl;
	m_nPlayPauseState = nState;
	if (nState == STATE_STOP) {
		m_pBoxTop->SetMessageWinState(MessageWindow::LOGO);
		m_pBoxTop->SwitchPlayButton(InterfaceTop::PLAY);
		m_nSourcePlaying = SourceTable::NONE;
	}
	else if (nState == STATE_PLAY) {
		m_pBoxTop->SetMessageWinState(MessageWindow::PLAY);
		m_pBoxTop->SwitchPlayButton(InterfaceTop::PAUSE);
		UpdateMessageWindow();
	}
	else if (nState == STATE_PAUSE) {
		m_pBoxTop->SwitchPlayButton(InterfaceTop::PLAY);
	}
}

//------------------------------------------------------------------------------

void
Interface::UpdateMessageWindow()
{
	xb::AutoLock lock(&s_ThreadLock, "UpdateMessageWindow");
	Assert(m_nCurrentFileIndex < m_listSelectedSongs.size(),
		   "current file index > selected list size");

	m_pBoxTop->SetMessageWinSong(m_listSelectedSongs[m_nCurrentFileIndex]);
}

//------------------------------------------------------------------------------

void
Interface::SelectRow(int nRow, bool bOn)
{
	m_pActiveList->SetCheckbox(nRow, bOn);
}

//------------------------------------------------------------------------------

void
Interface::CheckAllSongs()
{
	EM::I()->Trace() << "CheckAllSongs..." << endl;
	/*
	for (unsigned int i = 0; i < m_pActiveList->rows().size(); i++) {
		SelectRow(i, true);
	}
	*/
	m_pActiveList->CheckAll();
	EM::I()->Trace() << "CheckAllSongs...done" << endl;
}

//------------------------------------------------------------------------------

void
Interface::ButtonNewPlayList()
{
	System::GetController()->CreateNewPlayList();
	m_pSourceTable->UpdateSources(true);
}

//------------------------------------------------------------------------------

void
Interface::ButtonAction()
{
	if (m_nActionState == STATE_BROWSE) {
		// import song
		m_ImportDialog.show_all();
	}
	else if (m_nActionState == STATE_RIP) {
		if (System::GetController()->RipAvailable()) {
			m_nCurrentAction = STATE_RIP;
			if (System::GetPropList()->GetIntValue("ConfirmRip")) {
				m_Connection.disconnect();
				ConfirmDialog* pDialog = ConfirmDialog::Instance();
				m_Connection =
					pDialog->Done.connect(SigC::slot(this,
													 &Interface::RipConfirm));
				pDialog->SetText("Proceed with rip?");
				pDialog->CenterOverDisplay();
			}
			else {
				Rip(System::GetPropList()->GetBoolValue("RipDebug"));
			}
		}
		else {
			EM::I()->Trace() << "rip unavailable." << endl;
		}
	}
	else if (m_nActionState == STATE_BURN) {
		if (System::GetController()->BurnAvailable()) {
			m_nCurrentAction = STATE_BURN;
			if (System::GetPropList()->GetIntValue("ConfirmBurn")) {
				m_Connection.disconnect();
				ConfirmDialog* pDialog = ConfirmDialog::Instance();
				m_Connection =
					pDialog->Done.connect(SigC::slot(this,
													 &Interface::BurnConfirm));
				pDialog->SetText("Proceed with burn?");
				pDialog->CenterOverDisplay();
			}
			else {
				Burn(System::GetPropList()->GetBoolValue("BurnDebug"));
			}
		}
		else {
			EM::I()->Trace() << "burn unavailable." << endl;
		}
	}
}

//------------------------------------------------------------------------------

void
Interface::ButtonEject()
{
	System::GetController()->CDEject();
}

//------------------------------------------------------------------------------

void
Interface::ButtonVisualization()
{

}

//------------------------------------------------------------------------------

void
Interface::ButtonShrink()
{
	if (m_bShrunk) {
		set_usize(System::GetPropList()->GetIntValue("AppWidth"),
				  System::GetPropList()->GetIntValue("AppHeight"));
		m_pBoxMiddle->show();
		m_pBoxBottom->show();
		m_pBoxTop->Shrink(false);
		m_bShrunk = false;
	}
	else {
		m_pBoxMiddle->hide();
		m_pBoxBottom->hide();
		set_usize(500, 100);
		m_pBoxTop->Shrink(true);

		get_window().get_deskrelative_origin(m_nPosX, m_nPosY);
		get_window().move_resize(m_nPosX, m_nPosY, 480, 110);
		
		m_bShrunk = true;
	}

}

//------------------------------------------------------------------------------

void
Interface::ButtonConfig()
{
	ConfigDialog* pDialog = ConfigDialog::Instance();
	pDialog->show_all();
	pDialog->CenterOverDisplay();
}

//------------------------------------------------------------------------------

void
Interface::UpdateTimeLabel()
{
	EM::I()->Trace() << "UpdateTimeLabel" << endl;
	SongList listSongs;
	m_pActiveList->GetSelectedSongs(listSongs);

	long nSeconds = listSongs.GetTotalSeconds();
	off_t nSize = listSongs.GetTotalFileSize();

	if (nSize > 0) {
		m_labelTime.set(xb::String(listSongs.size()) + " songs, " +
						Util::GetTimeStr(nSeconds) + " total time, " +
						Util::GetFileSizeStr(nSize));
	}
	else {
		m_labelTime.set(xb::String(listSongs.size()) + " songs, " +
						Util::GetTimeStr(nSeconds) + " total time");
	}
}

//------------------------------------------------------------------------------

void
Interface::VolumeChanged()
{
	System::GetController()->SetVolume(m_pBoxTop->GetVolume());
}

//------------------------------------------------------------------------------

void
Interface::ActiveListReordered()
{
	PlayList* pPlayList = System::GetController()->GetPlayList(m_strSelectedSource);
	AssertNotNull(pPlayList);

	SongList listSongs;
	m_pActiveList->GetAllSongs(listSongs);
	pPlayList->SetSongList(listSongs);
	pPlayList->Save();
}

//------------------------------------------------------------------------------

int
Interface::ActiveListKey(GdkEventKey* pKey)
{
	AssertNotNull(pKey);

	// EM::I()->Trace("state: " + xb::String(pKey->state));
	// EM::I()->Trace("val: " + xb::String(pKey->keyval));

	if (pKey->keyval == GDK_Delete) {
		std::string strSongList;
		Gtk::CList_Helpers::SelectionList sslist = m_pActiveList->selection();
		for	(Gtk::CList_Helpers::SelectionList::const_iterator itr =
				 sslist.begin(); itr != sslist.end(); ++itr) {
			Song* pSong = (Song*)(itr->get_data());
			strSongList += pSong->GetTitle() + "\n";
		}
		
		if (m_nSelectedSourceType == SourceTable::PLAYLIST) {
			m_Connection.disconnect();
			ConfirmDialog* pDialog = ConfirmDialog::Instance();
			m_Connection =
				pDialog->Done.connect(SigC::slot(this,
												 &Interface::PlayListSongDelete));
			pDialog->SetText("Are you sure you want to delete\n" + strSongList +
							 " from playlist?");
			pDialog->CenterOverDisplay();
		}
		else if (m_nSelectedSourceType == SourceTable::LIBRARY) {
			m_Connection.disconnect();
			ConfirmDialog* pDialog = ConfirmDialog::Instance();
			m_Connection =
				pDialog->Done.connect(SigC::slot(this,
												 &Interface::LibrarySongDelete));
			pDialog->SetText("Are you sure you want to permanently delete\n" +
							 strSongList + " from the library?");
			pDialog->CenterOverDisplay();
		}
	}

	return 0;
}

//------------------------------------------------------------------------------

int
Interface::ActiveListClick(GdkEventButton* pButton)
{
	EM::I()->Trace() << "ActiveListClick:  " << pButton->button << endl;

	AssertNotNull(pButton);
	
	if (pButton->button == 3) {
		m_pActiveList->unselect_all();
		int nRow, nCol;
		m_pActiveList->get_selection_info(pButton->x, pButton->y, &nRow, &nCol);
		m_pActiveList->row(nRow).select();

		Song* pSong = (Song*)(m_pActiveList->row(nRow).get_data());
		if (pSong != NULL) {
			InfoDialog::Instance()->SetSong(*pSong, m_nSelectedSourceType);
			InfoDialog::Instance()->show_all();
		}
	}
	else if (pButton->button == 2) {
		int nRow, nCol;
		m_pActiveList->get_selection_info(pButton->x, pButton->y, &nRow, &nCol);
		m_pActiveList->row(nRow).select();
	}
	
	return 0;
}

//------------------------------------------------------------------------------

int
Interface::SourceTableKey(GdkEventKey* pKey)
{
	AssertNotNull(pKey);

	// EM::I()->Trace("state: " + xb::String(pKey->state));
	// EM::I()->Trace("val: " + xb::String(pKey->keyval));
	
	if (pKey->keyval == GDK_Delete) {
		EM::I()->Trace() << "delete pressed" << endl;
		if (m_nSelectedSourceType == SourceTable::PLAYLIST) {
			m_Connection.disconnect();
			ConfirmDialog* pDialog = ConfirmDialog::Instance();
			m_Connection =
				pDialog->Done.connect(SigC::slot(this,
												 &Interface::SourceTableDelete));
			pDialog->SetText("Are you sure you want to delete '" +
							 m_strSelectedSource + "' ?");
			pDialog->CenterOverDisplay();
		}
	}

	return 0;
}

//------------------------------------------------------------------------------

void
Interface::SourceTableDelete(bool bOK)
{
	m_Connection.disconnect();
	if (bOK) {
		EM::I()->Trace() << "deleting " << m_strSelectedSource << "..." << endl;
		System::GetController()->DeletePlayList(m_strSelectedSource);
		m_pSourceTable->UpdateSources(true);

		UpdateSourceSongList();
	}
}

//------------------------------------------------------------------------------

void
Interface::PlayListSongDelete(bool bOK)
{
	m_Connection.disconnect();
	if (bOK) {
		EM::I()->Trace() << "deleting playlist song..." << endl;
		SongList listSongs;
		Gtk::CList_Helpers::SelectionList sslist = m_pActiveList->selection();
		for	(Gtk::CList_Helpers::SelectionList::const_iterator itr =
				 sslist.begin(); itr != sslist.end(); ++itr) {
			Song* pSong = (Song*)(itr->get_data());
			listSongs.push_back(*pSong);
		}

		System::GetController()->DeletePlayListSongs(m_strSelectedSource,
													 listSongs);

		UpdateSourceSongList();
	}
}

//------------------------------------------------------------------------------

void
Interface::LibrarySongDelete(bool bOK)
{
	m_Connection.disconnect();
	if (bOK) {
		EM::I()->Trace() << "deleting library song..." << endl;
		SongList listSongs;
		Gtk::CList_Helpers::SelectionList sslist = m_pActiveList->selection();
		for	(Gtk::CList_Helpers::SelectionList::const_iterator itr =
				 sslist.begin(); itr != sslist.end(); ++itr) {
			Song* pSong = (Song*)(itr->get_data());
			listSongs.push_back(*pSong);
		}

		System::GetController()->DeleteLibrarySongs(listSongs);
		SetSongList(System::GetLibrary()->GetSongList());
	}

}

//------------------------------------------------------------------------------

void
Interface::Burn(bool bNoAction)
{
	if (System::GetController()->BurnAvailable()) {
		EM::I()->Trace() << "doing burn..." << endl;
		m_pBoxTop->SetMessageWinState(MessageWindow::PROGRESS);
		SongList listSongs;
		m_pActiveList->GetSelectedSongs(listSongs);
		System::GetController()->Burn(listSongs, bNoAction);
	}
	else {
		EM::I()->Trace() << "burn unavailable..." << endl;
	}
}
	
//------------------------------------------------------------------------------

void
Interface::BurnStatusUpdate(const std::string& strMsg)
{
	xb::AutoLockGtk lock("BurnStatusUpdate");
	m_pBoxTop->SetMessageWinProgressText(strMsg);
}

//------------------------------------------------------------------------------

void
Interface::BurnDone()
{
	xb::AutoLockGtk lock("BurnDone");
	m_pBoxTop->SetMessageWinProgressText("Burn complete.");
	m_nCurrentAction = STATE_BROWSE;
}

//------------------------------------------------------------------------------

void
Interface::BurnConfirm(bool bOK)
{
	if (bOK) {
		Burn(System::GetPropList()->GetBoolValue("BurnDebug"));
	}
}

//------------------------------------------------------------------------------

void
Interface::Rip(bool bNoAction)
{
	if (System::GetController()->RipAvailable()) {
		EM::I()->Trace() << "doing rip..." << endl;
		m_pBoxTop->SetMessageWinState(MessageWindow::PROGRESS);
		SongList listSongs;
		m_pActiveList->GetSelectedSongs(listSongs);
		System::GetController()->Rip(listSongs, bNoAction);
	}
	else {
		EM::I()->Trace() << "rip unavailable..." << endl;
	}
}

//------------------------------------------------------------------------------

void
Interface::RipStatusUpdate(const std::string& strMsg)
{
	xb::AutoLockGtk lock("RipStatusUpdate");
	EM::I()->Trace() << "thread enter: RipStatusUpdate" << endl;
	m_pBoxTop->SetMessageWinProgressText(strMsg);
	EM::I()->Trace() << "thread leave: RipStatusUpdate" << endl;
}

//------------------------------------------------------------------------------

void
Interface::RipDone()
{
	xb::AutoLockGtk lock("RipDone");
	EM::I()->Trace() << "done ripping." << endl;
	m_pBoxTop->SetMessageWinProgressText("Rip complete");
	System::GetController()->UpdatePlayListSystem();
	m_pSourceTable->UpdateSources(true);
	m_nCurrentAction = STATE_BROWSE;
}

//------------------------------------------------------------------------------

void
Interface::RipConfirm(bool bOK)
{
	if (bOK) {
		Rip(System::GetPropList()->GetBoolValue("RipDebug"));
	}
}

//------------------------------------------------------------------------------

void
Interface::MessageWinConfirmOK(bool bOK)
{
	if (m_nCurrentAction == STATE_BURN) {
		if (bOK) {
			Burn(System::GetPropList()->GetBoolValue("BurnDebug"));
		}
		else {
			m_pBoxTop->SetMessageWinState(MessageWindow::LOGO);
			m_nCurrentAction = STATE_BROWSE;
		}
	}
	else if (m_nCurrentAction == STATE_RIP) {
		if (bOK) {
			Rip(System::GetPropList()->GetBoolValue("RipDebug"));
		}
		else {
			m_pBoxTop->SetMessageWinState(MessageWindow::LOGO);
			m_nCurrentAction = STATE_BROWSE;
		}
	}
}

//------------------------------------------------------------------------------

void
Interface::ProgressCancel()
{
	if (m_nCurrentAction == STATE_BURN) {
		System::GetController()->BurnCancel();
		m_nCurrentAction = STATE_BROWSE;
		m_pBoxTop->SetMessageWinState(MessageWindow::LOGO);
	}
	else if (m_nCurrentAction == STATE_RIP) {
		System::GetController()->RipCancel();
		m_nCurrentAction = STATE_BROWSE;
		m_pBoxTop->SetMessageWinState(MessageWindow::LOGO);
	}
}

//------------------------------------------------------------------------------

void
Interface::MessageWinProgressClicked(float fPercentage)
{
	System::GetController()->PlayerSeek(fPercentage);
}

//------------------------------------------------------------------------------

void
Interface::SearchKey()
{
	EM::I()->Trace() << "search key pressed" << endl;
	if (m_pBoxTop->GetSearch() != m_strSearch) {
		EM::I()->Trace() << "search string new" << endl;
		m_strSearch = m_pBoxTop->GetSearch();
		UpdateSourceSongList();
	}
}

//------------------------------------------------------------------------------

void
Interface::UpdateSearchLabel(int nActiveCount, int nTotalCount)
{
	Assert(nActiveCount <= nTotalCount, "invalid song count");

	if (nActiveCount != nTotalCount) {
		m_pBoxTop->SetSearchLabel(xb::String(nActiveCount) + " of " + xb::String(nTotalCount));
	}
	else {
		m_pBoxTop->SetSearchLabel("Search");
	}
}

//------------------------------------------------------------------------------

void
Interface::PlayerStressTest()
{
	EM::I()->Trace() << "PlayerStressTest:" << endl;
	for (int i = 0; i < 100; i++) {
		System::GetController()->PlayerStop(false);
		PlaySelectedSongs(0);
	}
	System::GetController()->PlayerStop(false);
	EM::I()->Trace() << "PlayerStressTestDone." << endl;
}

//------------------------------------------------------------------------------

void
Interface::PlayerStressTestSeq()
{
	EM::I()->Trace() << "PlayerStressTestSeq:" << endl;
	PlaySelectedSongs(0);
	for (int i = 0; i < 100; i++) {
		System::GetController()->PlayerStop(true);
	}
	System::GetController()->PlayerStop(false);
	EM::I()->Trace() << "PlayerStressTestSeqDone." << endl;
}

//------------------------------------------------------------------------------

void
Interface::BurnStressTest()
{
	for (int i = 0; i < 100; i++) {
		Burn(true);
	}
}	

//------------------------------------------------------------------------------

void
Interface::RipStressTest()
{
	for (int i = 0; i < 100; i++) {
		Rip(true);
	}
}	

//------------------------------------------------------------------------------

void
Interface::ButtonImportOK()
{
	m_ImportDialog.hide_all();
	bool bLink = m_ImportDialog.GetLink();
	EM::I()->Trace() << "import filename:  " << m_ImportDialog.get_filename() <<
		", link == " << bLink << endl;

	/*
	Song song(m_ImportDialog.get_filename());
	EM::I()->Trace("import song dump:");
	song.Dump();

	System::GetLibrary()->Import(song, m_ImportDialog.get_filename(), bLink);
	*/

	System::GetLibrary()->Import(m_ImportDialog.get_filename(), bLink);
	
}

//------------------------------------------------------------------------------

int
Interface::Idle()
{
	static int nCDPollTime = 0;

	nCDPollTime++;
	if (nCDPollTime > 100) {
		nCDPollTime = 0;
		if (System::GetController()->CDMediaChanged()) {
			EM::I()->Trace() << "Interface::Idle() -- cd media changed" << endl;
			m_pSourceTable->UpdateCDInfo();
		}
	}
	
	if (System::GetController()->IsPlaying()) {
		int nTime = System::GetController()->GetPlayerElapsedTime();
		m_pBoxTop->SetTime(nTime);
		if (System::GetController()->IsPlayComplete()) {
			System::GetController()->PlayerStop(false);
			PlayComplete();
		}

		System::GetController()->UpdateVisualization(nTime);
	}

	return 1;
}

//------------------------------------------------------------------------------

int
Interface::TitlePress(GdkEventButton* pEvent)
{
	get_window().raise();

	m_nMotionX = (int)(pEvent->x_root);
	m_nMotionY = (int)(pEvent->y_root);

	get_window().get_deskrelative_origin(m_nPosX, m_nPosY);

	return 0;
}

//------------------------------------------------------------------------------

int
Interface::TitleMotion(GdkEventMotion* pEvent)
{
	get_window().raise();

	double fDeltaX = pEvent->x_root - m_nMotionX;
	if (fDeltaX > 0.0 && fDeltaX < 1.0) {
		fDeltaX = 1.0;
	}
	else if (fDeltaX < 0.0 && fDeltaX > -1.0) {
		fDeltaX = -1.0;
	}
	int nDeltaX = (int)fDeltaX;

	double fDeltaY = pEvent->y_root - m_nMotionY;
	if (fDeltaY > 0.0 && fDeltaY < 1.0) {
		fDeltaY = 1.0;
	}
	else if (fDeltaY < 0.0 && fDeltaY > -1.0) {
		fDeltaX = -1.0;
	}
	int nDeltaY = (int)fDeltaY;
	
	get_window().move(m_nPosX + nDeltaX, m_nPosY + nDeltaY);

	gint nX, nY;
	get_window().get_deskrelative_origin(nX, nY);
	m_nMotionX = nX - m_nPosX + m_nMotionX;
	m_nMotionY = nY - m_nPosY + m_nMotionY;
	m_nPosX = nX;
	m_nPosY = nY;

	return 0;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------



