// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Interface.h
//
// $Id: Interface.h,v 1.47 2002/02/18 21:00:30 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __INTERFACE__
#define __INTERFACE__

#include <gtk--/entry.h>
#include <gtk--/label.h>
#include <gtk--/main.h>
#include <gtk--/window.h>
#include <xb/ButtonImgToggle.h>

#include "ImportDialog.h"
#include "Song.h"
#include "SongList.h"
#include "SourceTable.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class ActiveList;
class InterfaceTop;
class MessageWindow;

//------------------------------------------------------------------------------

class Interface: public Gtk::Window
{
	// Constants
	private:
		enum {
			STATE_PLAY,
			STATE_PAUSE,
			STATE_STOP
		};

		enum {
			STATE_BURN,
			STATE_RIP,
			STATE_BROWSE
		};

	// Variables
	private:
		// top interface
		InterfaceTop* m_pBoxTop;

		// boxes
		Gtk::VBox* m_pBoxMaster;
		Gtk::HBox* m_pBoxMiddle;
		Gtk::HBox* m_pBoxBottom;
		
		// Buttons
		xb::ButtonImgToggle m_bnRandom;
		xb::ButtonImgToggle m_bnLoop;

		// Entries
		Gtk::Entry m_entrySource;

		// Labels
		Gtk::Label m_labelTime;

		// Lists
		ActiveList* m_pActiveList;
		SourceTable* m_pSourceTable;

		// File Selection
		ImportDialog m_ImportDialog;
		
		// Selection
		int m_nRowSelected;

		// State
		int m_nPlayPauseState;
		int m_nActionState;
		int m_nCurrentAction;

		unsigned int m_nCurrentFileIndex;
		std::string m_strSelectedSource;
		SourceTable::SourceType m_nSelectedSourceType;
		SourceTable::SourceType m_nSourcePlaying;
		SongList m_listSelectedSongs;
		SigC::Connection m_Connection;

		std::string m_strSearch;

		static pthread_mutex_t s_ThreadLock;

		bool m_bShrunk;

		int m_nMotionX, m_nMotionY;
		gint m_nPosX, m_nPosY;

		bool m_bDecorate;
		
	// Methods
    public:
		Interface();
		~Interface();

		virtual gint delete_event_impl(GdkEventAny* pEvent);

		void Init();

		void AppendSong(const Song& song);
		void SetSongList(const SongList& listSongs);

		void PlayComplete();
		void CheckAllSongs();
		
	private:
		void SongSelected(int nRow, int nCol, GdkEvent* pEvent);
		void SwitchState(int nState);
		void SelectRow(int nRow, bool bOn);
		void SetCheckbox(int nRow, bool bOn);
		void SourceSelected(int nRow, int nCol, GdkEvent* pEvent);
		void SourceEntryActivate();

		void ButtonPlayPause();
		void ButtonNext();
		void ButtonPrev();
		void ButtonNewPlayList();
		void ButtonAction();
		void ButtonShrink();
		void ButtonEject();
		void ButtonVisualization();
		void ButtonConfig();
		
		void PlaySelectedSongs(unsigned int nStartIndex);
		
		void UpdateMessageWindow();
		void UpdateSources();
		void UpdateTimeLabel();
		void UpdateSourceSongList();
		
		void VolumeChanged();

		void Rip(bool bNoAction);
		void RipStatusUpdate(const std::string& strMsg);
		void RipDone();
		void RipConfirm(bool bOK);

		void Burn(bool bNoAction);
		void BurnStatusUpdate(const std::string& strMsg);
		void BurnDone();
		void BurnConfirm(bool bOK);
		
		void ActiveListReordered();
		int ActiveListKey(GdkEventKey* pKey);
		int ActiveListClick(GdkEventButton* pButton);
		int SourceTableKey(GdkEventKey* pKey);
		void SourceTableDelete(bool bOK);

		void PlayListSongDelete(bool bOK);
		void LibrarySongDelete(bool bOK);

		void MessageWinConfirmOK(bool bOK);
		void ProgressCancel();
		void MessageWinProgressClicked(float fPercentage);
		
		void SearchKey();
		void UpdateSearchLabel(int nActiveCount, int nTotalCount);

		void PlayerStressTest();
		void PlayerStressTestSeq();
		void BurnStressTest();
		void RipStressTest();

		void ButtonImportOK();

		int Idle();

		int TitlePress(GdkEventButton* pEvent);
		int TitleMotion(GdkEventMotion* pEvent);
};

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
