// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// InterfaceTop.cpp
//
// $Id: InterfaceTop.cpp,v 1.21 2002/02/18 21:00:30 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include <config.h>

#include <gtk--/frame.h>
#include <gtk--/table.h>
#include <xb/ErrorManager.h>

#include "InterfaceTop.h"

#include "MessageWindow.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

const std::string InterfaceTop::s_strPlayNormalFile(
	INTERFACE_DATA_DIR "/button-play-normal.xpm");
const std::string InterfaceTop::s_strPlayOverFile(
	INTERFACE_DATA_DIR "/button-play-over.xpm");
const std::string InterfaceTop::s_strPlayDownFile(
	INTERFACE_DATA_DIR "/button-play-down.xpm");
const std::string InterfaceTop::s_strStopNormalFile(
	INTERFACE_DATA_DIR "/button-stop-normal.xpm");
const std::string InterfaceTop::s_strStopOverFile(
	INTERFACE_DATA_DIR "/button-stop-over.xpm");
const std::string InterfaceTop::s_strStopDownFile(
	INTERFACE_DATA_DIR "/button-stop-down.xpm");
const std::string InterfaceTop::s_strPauseNormalFile(
	INTERFACE_DATA_DIR "/button-pause-normal.xpm");
const std::string InterfaceTop::s_strPauseOverFile(
	INTERFACE_DATA_DIR "/button-pause-over.xpm");
const std::string InterfaceTop::s_strPauseDownFile(
	INTERFACE_DATA_DIR "/button-pause-down.xpm");
const std::string InterfaceTop::s_strImportNormalFile(
	INTERFACE_DATA_DIR "/wheelbarrow.xpm");
const std::string InterfaceTop::s_strImportOverFile(
	INTERFACE_DATA_DIR "/wheelbarrow-over.xpm");
const std::string InterfaceTop::s_strImportDownFile(
	INTERFACE_DATA_DIR "/wheelbarrow-down.xpm");
const std::string InterfaceTop::s_strBurnNormalFile(
	INTERFACE_DATA_DIR "/burn.xpm");
const std::string InterfaceTop::s_strBurnOverFile(
	INTERFACE_DATA_DIR "/burn-over.xpm");
const std::string InterfaceTop::s_strBurnDownFile(
	INTERFACE_DATA_DIR "/burn-down.xpm");
const std::string InterfaceTop::s_strRipNormalFile(
	INTERFACE_DATA_DIR "/rip.xpm");
const std::string InterfaceTop::s_strRipOverFile(
	INTERFACE_DATA_DIR "/rip-over.xpm");
const std::string InterfaceTop::s_strRipDownFile(
	INTERFACE_DATA_DIR "/rip-down.xpm");

//------------------------------------------------------------------------------

InterfaceTop::InterfaceTop() :
	Gtk::HBox(false, 0),
		 m_bnPlayPause(InterfaceTop::s_strPlayNormalFile,
					   InterfaceTop::s_strPlayOverFile,
					   InterfaceTop::s_strPlayDownFile),

	m_labelAction("Import"),
	m_labelSearch("Search"),
	m_adjVolume(50.0, 0.0, 100.0),
	m_scaleVolume(m_adjVolume),
		 m_bnAction(InterfaceTop::s_strImportNormalFile,
					InterfaceTop::s_strImportOverFile,
					InterfaceTop::s_strImportDownFile)
{
	// set_spacing(10);

	Gtk::VBox* pBoxPlayControls = manage(new Gtk::VBox(false, 0));
	pack_start(*pBoxPlayControls, false, false);

	m_pMessageWin = manage(new MessageWindow());
    pack_start(*m_pMessageWin, true, true, 30);

	m_pBoxSearch = manage(new Gtk::VBox(false, 0));
	pack_start(*m_pBoxSearch, true, false);

	m_pBoxState = manage(new Gtk::VBox(false, 0));
	pack_start(*m_pBoxState, true, false);

	xb::ButtonImg* pButtonPrev =
		manage(new xb::ButtonImg(INTERFACE_DATA_DIR "/button-prev-normal.xpm",
								 INTERFACE_DATA_DIR "/button-prev-over.xpm",
								 INTERFACE_DATA_DIR "/button-prev-down.xpm"));

	xb::ButtonImg* pButtonNext =
		manage(new xb::ButtonImg(INTERFACE_DATA_DIR "/button-next-normal.xpm",
								 INTERFACE_DATA_DIR "/button-next-over.xpm",
								 INTERFACE_DATA_DIR "/button-next-down.xpm"));

	
	Gtk::Table* pTable = manage(new Gtk::Table(1, 3));
	pTable->attach(*pButtonPrev, 0, 1, 0, 1, GTK_FILL, GTK_FILL);
	pTable->attach(m_bnPlayPause, 1, 2, 0, 1, GTK_FILL, GTK_FILL);
	pTable->attach(*pButtonNext, 2, 3, 0, 1, GTK_FILL, GTK_FILL);
	pBoxPlayControls->pack_start(*pTable, false, false);
	
	pButtonPrev->set_usize(48, 48);
	m_bnPlayPause.set_usize(48, 48);
	pButtonNext->set_usize(48, 48);
	
	m_scaleVolume.set_draw_value(0);
	Gtk::HBox* pBoxVolume = manage(new Gtk::HBox(/*false, 0???*/));
	pBoxVolume->set_homogeneous(0);
	pBoxVolume->set_spacing(5);
	pBoxVolume->set_border_width(5);

	Gtk::Pixmap* pPixSoft =
		manage(new Gtk::Pixmap(INTERFACE_DATA_DIR "/volume-low.xpm"));
	Gtk::Pixmap* pPixLoud =
		manage(new Gtk::Pixmap(INTERFACE_DATA_DIR "/volume-hi.xpm"));
	pBoxVolume->pack_start(*pPixSoft, false, false);
	pBoxVolume->pack_start(m_scaleVolume);
	pBoxVolume->pack_start(*pPixLoud, false, false);
	pBoxPlayControls->pack_start(*pBoxVolume, false, false);
	
	// - Search box contents
	m_pBoxSearch->pack_start(m_entrySearch, true, false);
	// Gtk::Label* pLabelSearch = manage(new Gtk::Label("Search"));
	// m_pBoxSearch->pack_start(*pLabelSearch, false, false);
	m_pBoxSearch->pack_start(m_labelSearch, true, false);

	// - State box contents
	// m_pBoxState->pack_start(m_pixBurn, false, false);
	// m_bnAction.add(m_labelAction);
	m_pBoxState->pack_start(m_bnAction, false, false);
	m_pBoxState->pack_start(m_labelAction);

	/*
	Gtk::Table* pBTable = manage(new Gtk::Table(1, 1));
	xb::ButtonImg* pButtonShrink =
		manage(new xb::ButtonImg(INTERFACE_DATA_DIR "/shrink-up.xpm",
							 INTERFACE_DATA_DIR "/shrink-over.xpm",
							 INTERFACE_DATA_DIR "/shrink-down.xpm"));
	pBTable->attach(*pButtonShrink, 0, 1, 0, 1, GTK_FILL, GTK_FILL);
	pack_start(*pBTable, false, false);
	*/
	

	
	// signals
	m_bnPlayPause.clicked.connect(ButtonPlayPause.slot());
	pButtonPrev->clicked.connect(ButtonPrev.slot());
	pButtonNext->clicked.connect(ButtonNext.slot());
	m_bnAction.clicked.connect(ButtonAction.slot());
	m_entrySearch.changed.connect(SearchKey.slot());
	m_pMessageWin->ProgressCancel.connect(MessageWinProgressCancel.slot());
	m_pMessageWin->ProgressClicked.connect(MessageWinProgressClicked.slot());
	m_adjVolume.value_changed.connect(VolumeChanged.slot());
	// pButtonShrink->clicked.connect(ButtonShrink.slot());
}

//------------------------------------------------------------------------------

InterfaceTop::~InterfaceTop()
{

}

//------------------------------------------------------------------------------

void
InterfaceTop::SetVolume(int nVal)
{
	m_adjVolume.set_value(nVal);
}

//------------------------------------------------------------------------------

void
InterfaceTop::SetActionLabel(const std::string& strText)
{
	m_labelAction.set(strText);
}

//------------------------------------------------------------------------------

void
InterfaceTop::SetActionType(ActionType nType)
{
	if (nType == IMPORT) {
		SetActionLabel("Import");
		m_bnAction.SetFiles(InterfaceTop::s_strImportNormalFile,
							InterfaceTop::s_strImportOverFile,
							InterfaceTop::s_strImportDownFile);
	}
	else if (nType == RIP) {
		SetActionLabel("Rip");
		m_bnAction.SetFiles(InterfaceTop::s_strRipNormalFile,
							InterfaceTop::s_strRipOverFile,
							InterfaceTop::s_strRipDownFile);
	}
	else if (nType == BURN){
		SetActionLabel("Burn");
		m_bnAction.SetFiles(InterfaceTop::s_strBurnNormalFile,
							InterfaceTop::s_strBurnOverFile,
							InterfaceTop::s_strBurnDownFile);
	}
}

//------------------------------------------------------------------------------

void
InterfaceTop::SetMessageWinState(MessageWindow::State nState)
{
	AssertNotNull(m_pMessageWin);
	m_pMessageWin->SetState(nState);
}

//------------------------------------------------------------------------------

void
InterfaceTop::SetMessageWinSong(const Song& song)
{
	AssertNotNull(m_pMessageWin);
	m_pMessageWin->SetSong(song);
}

//------------------------------------------------------------------------------

void
InterfaceTop::SetMessageWinProgressText(const std::string& strText)
{
	AssertNotNull(m_pMessageWin);
	m_pMessageWin->SetProgressText(strText);
}

//------------------------------------------------------------------------------

void
InterfaceTop::SetSearchLabel(const std::string& strText)
{
	m_labelSearch.set(strText);
}

//------------------------------------------------------------------------------

std::string
InterfaceTop::GetSearch() const
{
	return m_entrySearch.get_text();
}

//------------------------------------------------------------------------------

gfloat
InterfaceTop::GetVolume() const
{
	return m_adjVolume.get_value();
}

//------------------------------------------------------------------------------

void
InterfaceTop::SwitchPlayButton(PlayButtonType nType)
{
	if (nType == PLAY) {
		m_bnPlayPause.SetFiles(InterfaceTop::s_strPlayNormalFile,
							   InterfaceTop::s_strPlayOverFile,
							   InterfaceTop::s_strPlayDownFile);
	}
	else if (nType == STOP) {
		m_bnPlayPause.SetFiles(InterfaceTop::s_strStopNormalFile,
							   InterfaceTop::s_strStopOverFile,
							   InterfaceTop::s_strStopDownFile);
	}
	else if (nType == PAUSE) {
		m_bnPlayPause.SetFiles(InterfaceTop::s_strPauseNormalFile,
							   InterfaceTop::s_strPauseOverFile,
							   InterfaceTop::s_strPauseDownFile);
	}
}

//------------------------------------------------------------------------------

void
InterfaceTop::Shrink(bool bIn)
{
	if (bIn) {
		m_pBoxState->hide();
		m_pBoxSearch->hide();
	}
	else {
		m_pBoxState->show();
		m_pBoxSearch->show();
	}
}

//------------------------------------------------------------------------------

void
InterfaceTop::SetTime(int nMS)
{
	AssertNotNull(m_pMessageWin);
	m_pMessageWin->SetTime(nMS);
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------

