// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// InterfaceTop.h
//
// $Id: InterfaceTop.h,v 1.16 2002/02/18 21:00:30 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __InterfaceTop__
#define __InterfaceTop__

#include <gtk--/adjustment.h>
#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/entry.h>
#include <gtk--/label.h>
#include <gtk--/pixmap.h>
#include <gtk--/scale.h>
#include <string>
#include <xb/ButtonImg.h>

#include "MessageWindow.h"
#include "Song.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class MessageWindow;

class InterfaceTop : public Gtk::HBox
{
	// Constants
	private:
		static const std::string s_strPlayNormalFile;
		static const std::string s_strPlayOverFile;
		static const std::string s_strPlayDownFile;
		static const std::string s_strStopNormalFile;
		static const std::string s_strStopOverFile;
		static const std::string s_strStopDownFile;
		static const std::string s_strPauseNormalFile;
		static const std::string s_strPauseOverFile;
		static const std::string s_strPauseDownFile;
		static const std::string s_strImportNormalFile;
		static const std::string s_strImportOverFile;
		static const std::string s_strImportDownFile;
		static const std::string s_strBurnNormalFile;
		static const std::string s_strBurnOverFile;
		static const std::string s_strBurnDownFile;
		static const std::string s_strRipNormalFile;
		static const std::string s_strRipOverFile;
		static const std::string s_strRipDownFile;

	public:
		enum ActionType {
			IMPORT,
			BURN,
			RIP
		};

		enum PlayButtonType {
			PLAY,
			STOP,
			PAUSE
		};
		
	// Types
	// Variables
	private:
		Gtk::VBox* m_pBoxSearch;
		Gtk::VBox* m_pBoxState;
		xb::ButtonImg m_bnPlayPause;
		// Gtk::Button m_bnAction;
		Gtk::Label m_labelAction;
		Gtk::Label m_labelSearch;
		// Scales
		Gtk::Adjustment m_adjVolume;
		Gtk::HScale m_scaleVolume;
		// Message Window
		MessageWindow* m_pMessageWin;
		// Entries
		Gtk::Entry m_entrySearch;
		// Pixmaps
		// Gtk::Pixmap m_pixBurn;
		xb::ButtonImg m_bnAction;

	// Methods
	public:
		InterfaceTop();
		~InterfaceTop();

		void SetVolume(int nVal);
		void SetActionType(ActionType nType);
		void SetMessageWinState(MessageWindow::State nState);
		void SetMessageWinSong(const Song& song);
		void SetMessageWinProgressText(const std::string& strText);
		void SetSearchLabel(const std::string& strText);

		void SwitchPlayButton(PlayButtonType nType);

		void Shrink(bool bIn);
		
		std::string GetSearch() const;
		gfloat GetVolume() const;

		void SetTime(int nMS);

		SigC::Signal0<void> ButtonPlayPause;
		SigC::Signal0<void> ButtonPrev;
		SigC::Signal0<void> ButtonNext;
		SigC::Signal0<void> ButtonAction;
		SigC::Signal0<void> ButtonShrink;

		SigC::Signal1<void, bool> MessageWinConfirmOK;
		SigC::Signal0<void> MessageWinProgressCancel;
		SigC::Signal1<void, float> MessageWinProgressClicked;

		SigC::Signal0<void> SearchKey;
		SigC::Signal0<void> VolumeChanged;

	private:
		void SetActionLabel(const std::string& strText);

};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif 

//------------------------------------------------------------------------------
