// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MWState.cpp
//
// $Id: MWState.cpp,v 1.3 2002/02/10 21:45:18 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include <config.h>

#include "MWState.h"

#include <gtk--/style.h>

#include "WidgetMsgWin.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

MWState::MWState(WidgetMsgWin* pParent) :
	m_pParent(pParent),
	m_nScrollX(0),
	m_nStep(1),
	m_nProgressWidth(0),
	m_nProgressHeight(10),
	m_nProgressX(15),
	m_nProgressY(32),
	m_pixEQButton(INTERFACE_DATA_DIR "/eq-button.xpm"),
	m_nEQButtonX(0),
	m_nEQButtonY(m_nProgressY - 1)
{
}

//------------------------------------------------------------------------------

MWState::~MWState()
{

}

//------------------------------------------------------------------------------

void
MWState::DrawCenteredString(Gdk_Pixmap* pPixmap, const Gdk_GC& gc,
							std::string s)
{
	Gdk_Font font;
	GetFont(font);

	int nMaxWidth = m_pParent->width();
	int nFontWidth = font.string_width(s);
	int nX = 0;
	if (nFontWidth < nMaxWidth) {
		nX = nMaxWidth / 2 - nFontWidth / 2;
	}

	int nMaxHeight = m_pParent->height();
	int nFontHeight = font.string_height(s);
	int nY = 0;
	if (nFontHeight < nMaxHeight) {
		nY = nMaxHeight / 2 - nFontHeight / 2;
	}

	pPixmap->draw_string(font, gc, nX, nY, s);
}

//------------------------------------------------------------------------------

void
MWState::DrawCenteredHString(Gdk_Pixmap* pPixmap, const Gdk_GC& gc,
							 std::string s, int nY, int nPadX)
{
	Gdk_Font font;
	GetFont(font);

	int nMaxWidth = m_pParent->width() - 2 * nPadX;
	int nFontWidth = font.string_width(s);
	int nX = 0;
	if (nFontWidth < nMaxWidth) {
		nX = nMaxWidth / 2 - nFontWidth / 2;
	}

	pPixmap->draw_string(font, gc, nX + nPadX, nY, s);
}

//------------------------------------------------------------------------------

void
MWState::DrawCenteredScrollString(Gdk_Pixmap* pPixmap, const Gdk_GC& gc,
								  const std::string& s, int nY)
{
	Gdk_Font font;
	GetFont(font);
	
	int nFontWidth = font.string_width(s);
	if (nFontWidth < m_pParent->width()) {
		DrawCenteredHString(pPixmap, gc, s, nY, 0);
	}
	else {
		pPixmap->draw_string(font, gc, m_nScrollX, nY, s);


		int nMinimum = 1;
		int nMaximum = 5;
		float fMultiplier = 0.4;
		
		// calculate sliding
		if (m_nStep < 0) {
			// moving left
			if (m_nScrollX < m_pParent->width() - nFontWidth) {
				m_nStep = -m_nStep;
			}
			else {
				int nDistance = nFontWidth - m_pParent->width()+ m_nScrollX;
				if (nDistance > nMinimum) {
					m_nStep = (int)(-fMultiplier * nDistance);
					if (m_nStep < -nMaximum) {
						m_nStep = -nMaximum;
					}
					else if (m_nStep > -nMinimum) {
						m_nStep = -nMinimum;
					}
				}
				else {
					m_nStep = -nMinimum;
				}
			}
		}
		else {
			// moving right
			if (m_nScrollX > 0) {
				m_nStep = -m_nStep;
			}
			else {
				int nDistance = -m_nScrollX;
				if (nDistance > nMinimum) {
					m_nStep = (int)(fMultiplier * nDistance);
					if (m_nStep > nMaximum) {
						m_nStep = nMaximum;
					}
					else if (m_nStep < nMinimum) {
						m_nStep = nMinimum;
					}
				}
				else {
					m_nStep = nMinimum;
				}
			}
		}

		m_nScrollX += m_nStep;
	}
}

//------------------------------------------------------------------------------

void
MWState::GetFont(Gdk_Font& font)
{
	font.create("-*-verdana-medium-r-*--*-80-*-*-*-*-*-*");
	if (font.gdkobj() == NULL) {
		font.create("-*-arial-medium-r-*--*-80-*-*-*-*-*-*");
		if (font.gdkobj() == NULL) {
			font.create("-*-*-medium-r-*--*-80-*-*-*-*-*-*");
			if (font.gdkobj() == NULL) {
				font.create("-*-*-*-*-*--*-80-*-*-*-*-*-*");
				if (font.gdkobj() == NULL) {
					font.create("-*-*-*-*-*--*-*-*-*-*-*-*-*");
				}
			}
		}
	}
}

//------------------------------------------------------------------------------

void
MWState::ResetScroll()
{
	m_nStep = 1;
	m_nScrollX = 0;
}

//------------------------------------------------------------------------------

void
MWState::DrawProgressBar(Gdk_Pixmap* pPixmap, float fPercentage)
{
	int nPadX = 15;
	m_nProgressWidth = m_pParent->width() - 2 * nPadX;
	DrawProgressBar(pPixmap, m_nProgressX, m_nProgressY, m_nProgressWidth,
					fPercentage);
}

//------------------------------------------------------------------------------

void
MWState::DrawProgressBar(Gdk_Pixmap* pPixmap, int nX, int nY, int nWidth,
						 float fPercent)
{
	pPixmap->draw_rectangle(m_pParent->get_style()->get_black_gc(), false,
							nX, nY, nWidth, m_nProgressHeight);

	int nSliderDim = m_nProgressHeight - 1;
	// subtract rect border width and width of slider
	int nSliderWidth = nWidth - nSliderDim; 
	int nSliderXPos = (int)(fPercent * nSliderWidth) + nX + 1;
	int nSliderYPos = nY + 1;

	pPixmap->draw_rectangle(m_pParent->get_style()->get_black_gc(), true,
							 nSliderXPos, nSliderYPos, nSliderDim,
							 nSliderDim);
}

//------------------------------------------------------------------------------

void
MWState::DrawEQButton(Gdk_Pixmap* pPixmap)
{
	Gdk_GC gc(*pPixmap);

	Gdk_Pixmap pix = m_pixEQButton.GetPixmap();
	Gdk_Bitmap mask = m_pixEQButton.GetMask();
	
	gc.set_clip_mask(mask);
	gc.set_clip_origin(m_nEQButtonX, m_nEQButtonY);
	
	pPixmap->draw_pixmap(gc, pix, 0, 0, m_nEQButtonX, m_nEQButtonY);
}

//------------------------------------------------------------------------------

void
MWState::SetColor(Gdk_GC& gc, int nRed, int nGreen, int nBlue)
{
	Gdk_Color color;
	Gdk_Colormap colormap(Gdk_Colormap::get_system());
	color.set_red(nRed);
	color.set_green(nGreen);
	color.set_blue(nBlue);
	colormap.alloc(color);
	gc.set_foreground(color);
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
