// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MWStateEQ.h
//
// $Id: MWStateEQ.h,v 1.4 2002/02/07 01:48:48 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __MWStateEQ__
#define __MWStateEQ__

#include "MWState.h"

#include <glib.h>

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class WidgetMsgWin;

class MWStateEQ : public MWState
{
	// Constants
	private:
		static const int s_nBarWidth;
		static const int s_nBarPad;
		static const int s_anXScale[21];
		static const int s_nXScaleMax = 19;
		static const double s_fYScale;

	// Types
	private:
		struct Cap {
				float fPercent;
				float fTime;
		};
	// Variables
	private:
		Cap m_aCaps[s_nXScaleMax];
		GTimer* m_pTimer;
		
	// Methods
	public:
		MWStateEQ(WidgetMsgWin* pParent);
		~MWStateEQ();

		virtual void Draw(Gdk_Pixmap* pPixmap);
		virtual bool NeedsDrawTimer() { return true; }
		virtual void ButtonPress(int nX, int nY);

		SigC::Signal0<void> EQButtonClicked;

	private:
		int GetPadX() const;
		void GetBarHeightY(float fPercent, int& nY, int& nHeight);
		void DrawBar(Gdk_Pixmap* pPixmap, int nX, float fPercent);
		void DrawCaps(Gdk_Pixmap* pPixmap);
		void CalcFreq(gint16 *dest, const gint16 *src);
		void CalcMonoFreq(gint16* dest, const gint16* src, gint nch);

		void SetCap(int nX, float fPercent);
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
