// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MWStatePlay.h
//
// $Id: MWStatePlay.h,v 1.5 2002/02/02 18:15:14 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __MWStatePlay__
#define __MWStatePlay__

#include "MWState.h"

#include <glib.h>

// #include "Image.h"
// #include "Pixmap.h"
#include "Song.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class WidgetMsgWin;

class MWStatePlay : public MWState
{
	// Constants
	private:
		static const int s_nBackColorRed;
		static const int s_nBackColorGreen;
		static const int s_nBackColorBlue;

		enum TextState {
			NORMAL,
			FADE_OUT,
			FADE_IN
		};

		enum TitleType {
			SONGTITLE,
			ARTIST,
			ALBUM,
			NUM_TITLETYPES
		};

	// Types
	// Variables
	private:
		int m_nMS;
		Song m_Song;

		TextState m_nTextState;
		int m_nTextRed;
		int m_nTextGreen;
		int m_nTextBlue;
		int m_nTimeCount;
		std::string m_strTitleText;
		TitleType m_nTitleType;
		GTimer* m_pTimer;

	// Methods
	public:
		MWStatePlay(WidgetMsgWin* pParent);
		~MWStatePlay();

		virtual void Draw(Gdk_Pixmap* pPixmap);
		virtual bool NeedsDrawTimer() { return true; }
		virtual void ButtonPress(int nX, int nY);

		void SetSong(const Song& song);
		void SetTime(int nMS) { m_nMS = nMS; }

		SigC::Signal1<void, float> ProgressClicked;
		SigC::Signal0<void> EQButtonClicked;
		
	private:
		void DrawTitle(Gdk_Pixmap* pPixmap);
		void DrawTime(Gdk_Pixmap* pPixmap, const std::string& strTime);
		void ResetFade();
		int FadeColor(int nSource, int nDest);

};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
