// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MessageWindow.cpp
//
// $Id: MessageWindow.cpp,v 1.16 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include <config.h>

#include "MessageWindow.h"

#include <gtk--/pixmap.h>
#include <xb/ErrorManager.h>

#include "MWStateEQ.h"
#include "MWStateLogo.h"
#include "MWStatePlay.h"
#include "MWStateProgress.h"
#include "WidgetMsgWin.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

MessageWindow::MessageWindow() :
	Gtk::Table(1, 3),
		 m_nState(NONE),
		 m_pMsgWin(NULL),
		 m_pMWStateEQ(NULL),
		 m_pMWStateLogo(NULL),
		 m_pMWStatePlay(NULL),
		 m_pMWStateProgress(NULL)
{
	Gtk::Pixmap* pPixmap = manage(
		new Gtk::Pixmap(INTERFACE_DATA_DIR "/mwin-left.xpm"));
	attach(*pPixmap, 0, 1, 0, 1, GTK_FILL, GTK_FILL);
	
	m_pMsgWin = manage(new WidgetMsgWin(200, 69));
	attach(*m_pMsgWin, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0);

	pPixmap = manage(
		new Gtk::Pixmap(INTERFACE_DATA_DIR "/mwin-right.xpm"));
	attach(*pPixmap, 2, 3, 0, 1, GTK_FILL, GTK_FILL);

	m_pMWStateEQ.reset(new MWStateEQ(m_pMsgWin));
	m_pMWStateLogo.reset(new MWStateLogo(m_pMsgWin));
	m_pMWStatePlay.reset(new MWStatePlay(m_pMsgWin));
	m_pMWStateProgress.reset(new MWStateProgress(m_pMsgWin));

	m_pMWStateProgress->CancelClicked.connect(ProgressCancel.slot());
	m_pMWStatePlay->ProgressClicked.connect(ProgressClicked.slot());
	m_pMWStatePlay->EQButtonClicked.connect(
		slot(this, &MessageWindow::PlayEQButtonClicked));
	m_pMWStateEQ->EQButtonClicked.connect(
		slot(this, &MessageWindow::EQEQButtonClicked));
}

//------------------------------------------------------------------------------

MessageWindow::~MessageWindow()
{
	// delete m_pMWStateEQ;
	// delete m_pMWStateLogo;
	// delete m_pMWStatePlay;
	// delete m_pMWStateProgress;
}

//------------------------------------------------------------------------------

void
MessageWindow::SetState(MessageWindow::State nState)
{
	AssertNotNull(m_pMsgWin);
	m_nState = nState;
//	m_pMsgWin->SetState(nState);
	if (nState == EQ) {
		EM::I()->Trace() << "MessageWindow::SetState() -- EQ" << endl;
		m_pMsgWin->SetState(m_pMWStateEQ.get());
	}
	else if (nState == LOGO) {
		EM::I()->Trace() << "MessageWindow::SetState() -- LOGO" << endl;
		m_pMsgWin->SetState(m_pMWStateLogo.get());
	}
	else if (nState == PLAY) {
		EM::I()->Trace() << "MessageWindow::SetState() -- PLAY" << endl;
		m_pMsgWin->SetState(m_pMWStatePlay.get());
	}
	else if (nState == PROGRESS) {
		EM::I()->Trace() << "MessageWindow::SetState() -- PROGRESS" << endl;
		m_pMsgWin->SetState(m_pMWStateProgress.get());
	}
	else {
		EM::I()->Trace() << "MessageWindow::SetState() -- NULL (" << nState <<
			")" << endl;
		m_pMsgWin->SetState(NULL);
	}
}

//------------------------------------------------------------------------------

void
MessageWindow::SetProgressText(const std::string& strText)
{
	AssertNotNull(m_pMWStateProgress.get());
	m_pMWStateProgress->SetProgressText(strText);
}

//------------------------------------------------------------------------------

void
MessageWindow::SetSong(const Song& song)
{
	AssertNotNull(m_pMWStatePlay.get());
	m_pMWStatePlay->SetSong(song);
}

//------------------------------------------------------------------------------

void
MessageWindow::SetTime(int nMS)
{
	AssertNotNull(m_pMWStatePlay.get());
	m_pMWStatePlay->SetTime(nMS);
}


//------------------------------------------------------------------------------

void
MessageWindow::PlayEQButtonClicked()
{
	SetState(EQ);
}

//------------------------------------------------------------------------------

void
MessageWindow::EQEQButtonClicked()
{
	SetState(PLAY);
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
