// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Mixer.cpp
//
// $Id: Mixer.cpp,v 1.13 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "Mixer.h"

#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/soundcard.h>
/*
#ifdef HAVE_SYS_SOUNDCARD_H
#include <sys/soundcard.h>
#elif defined(HAVE_MACHINE_SOUNDCARD_H)
#include <machine/soundcard.h>
#endif
*/
#include <unistd.h>
#include <xb/ErrorManager.h>

#include "PropList.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

Mixer::Mixer()
{
	m_strDevice = System::GetPropList()->GetValue("MixerDevice");
}

//------------------------------------------------------------------------------

Mixer::~Mixer()
{

}

//------------------------------------------------------------------------------

void
Mixer::SetVolume(int n)
{
	SetVolume(n, n);
}

//------------------------------------------------------------------------------

void
Mixer::SetVolume(int l, int r)
{
	int fd, v, cmd, devs;

	fd = open(m_strDevice.c_str(), O_RDONLY);

	if (fd != -1)
	{
		ioctl(fd, SOUND_MIXER_READ_DEVMASK, &devs);

		// PATRICK: ????
		bool bUseMaster = true;
		
		if ((devs & SOUND_MASK_PCM) && !bUseMaster)
			cmd = SOUND_MIXER_WRITE_PCM;
		else if ((devs & SOUND_MASK_VOLUME) && bUseMaster)
			cmd = SOUND_MIXER_WRITE_VOLUME;
		else
		{
			close(fd);
			return;
		}
		v = (r << 8) | l;
		ioctl(fd, cmd, &v);
		close(fd);
	}
	else {
		EM::I()->Error() << "SetVolume: Failed to open mixer device (" <<
			m_strDevice << "): " << strerror(errno) << endl;
	}
}

//------------------------------------------------------------------------------

int
Mixer::GetVolume() const
{
	int l, r;
	GetVolume(l, r);
	return (l + r) / 2;
}

//------------------------------------------------------------------------------

void
// Mixer::GetVolume(int* l, int* r) const
Mixer::GetVolume(int& l, int& r) const
{
	int fd, v, cmd, devs;

	fd = open(m_strDevice.c_str(), O_RDONLY);

	/*
	 * We dont show any errors if this fails, as this is called
	 * rather often
	 */
	if (fd != -1)
	{
		ioctl(fd, SOUND_MIXER_READ_DEVMASK, &devs);


		// PATRICK: ????
		bool bUseMaster = true;
		

		if ((devs & SOUND_MASK_PCM) && !bUseMaster)
			cmd = SOUND_MIXER_READ_PCM;
		else if ((devs & SOUND_MASK_VOLUME) && bUseMaster)
			cmd = SOUND_MIXER_READ_VOLUME;
		else
		{
			close(fd);
			return;
		}
		ioctl(fd, cmd, &v);
		r = (v & 0xFF00) >> 8;
		l = (v & 0x00FF);
		close(fd);
	}
}

//------------------------------------------------------------------------------

bool
Mixer::CheckCapability(std::string& strMessage)
{
	bool bResult = true;
	int fd = open(m_strDevice.c_str(), O_RDONLY);
	if (fd == -1) {
		bResult = false;
		strMessage = "Couldn't open mixer device: " + m_strDevice;
	}
	else {
		close(fd);
	}
	
	return bResult;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
