// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Output.h
//
// $Id: Output.h,v 1.5 2002/02/17 01:03:50 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __Output__
#define __Output__

#include <glib.h>
#include <string>

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class Output
{
	// Constants
	public:
		enum AFormat {
			 FMT_U8,
			 FMT_S8,
			 FMT_U16_LE,
			 FMT_U16_BE,
			 FMT_U16_NE,
			 FMT_S16_LE,
			 FMT_S16_BE,
			 FMT_S16_NE
		};
		
	// Types
	// Variables
	protected:
		bool m_bRealtime;
		
	// Methods
	public:
		Output();
		virtual ~Output();

		virtual void Init() = 0;

		virtual void GetVolume(int& nLeft, int& nRight) = 0;
		virtual void SetVolume(int nLeft, int nRight) = 0;

		virtual bool Open(AFormat nFormat, int nRate, int nChannels,
						  int nTrack) = 0;
		virtual void Write(void* pData, int nLength) = 0;
		virtual void Close() = 0;
		virtual void Flush(int nTime) = 0;
		virtual void Pause(bool bPause) = 0;

		virtual bool BufferPlaying() = 0;
		virtual int ElapsedTime() = 0;
		virtual guint64 WrittenTime() const = 0;
		virtual int BufferFreeAmt() = 0;

		virtual bool IsFileOutput() const = 0;
		virtual std::string GetFileExtension() const = 0;

		virtual std::string GetLibName() const = 0;
		virtual std::string GetDescription() const = 0;
		
	private:
		bool CheckRealtimePriority();
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
