// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// PlayListManager.h
//
// $Id: PlayListManager.h,v 1.9 2002/02/18 08:55:59 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __PlayListManager__
#define __PlayListManager__

#include <string>
#include <xb/FileUtil.h>
#include <xb/Filename.h>

#include "PlayList.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class PlayListManager
{
	// Constants
	// Types
	public:
		typedef vector<PlayList> PlayListVec;
		
	// Variables
	private:
		std::string m_strRoot;
		xb::FileUtil::DirectoryList m_listFiles;
		PlayListVec m_vecPlayLists;
		static pthread_mutex_t s_ThreadLock1;
		static pthread_mutex_t s_ThreadLock2;
		static pthread_mutex_t s_ThreadLock3;
		
	// Methods
	public:
		PlayListManager(const std::string& strRoot);
		~PlayListManager();

		void UpdateDirectoryList();
		void Dump() const;

		std::string GetRoot() const { return m_strRoot; }
		PlayList* GetPlayList(const std::string& strName);
		PlayList* GetPlayListFromFile(const std::string& strFilename);
		
		xb::FileUtil::DirectoryListConstItr ItrBegin() { return m_listFiles.begin(); }
		xb::FileUtil::DirectoryListConstItr ItrEnd() { return m_listFiles.end(); }

		const PlayListVec& GetPlayListsVec() const { return m_vecPlayLists; }
		
		void Rename(const std::string& strOriginal,
					const std::string& strNew);

		void DeletePlayList(const std::string& strName);
		
		PlayList* NewPlayList();
		PlayList* NewPlayList(const std::string& strName);
		PlayList* NewPlayList(const std::string& strName,
							  const SongList& listSongs);
		
	private:
		xb::Filename GetFilename(const std::string& strName) const;
		static bool Compare(const PlayList& p1, const PlayList& p2);
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
