// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Player.h
//
// $Id: Player.h,v 1.16 2002/02/18 08:55:59 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __PLAYER__
#define __PLAYER__

//------------------------------------------------------------------------------

#include <sigc++/signal_system.h>
#include <sigc++/object.h>
#include <string>
#include <vector>
#include <xb/Filename.h>

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class Output;
class Visualization;

class Player : public SigC::Object
{
	// Constants
	// Types
	public:
		typedef vector<std::string> FileList;
		typedef FileList::iterator FileListItr;
		typedef FileList::const_iterator FileListConstItr;
		
	// Variables
	protected:
		bool m_bPlaying;
		bool m_bPlayComplete;
		Output* m_pOutput;
		Visualization* m_pVisualization;
		
	// Methods
	public:
		Player();
		virtual ~Player();

		virtual void Init(Output* pOutput, Visualization* pVis);
		
		virtual void Play(const xb::Filename& strFilename, int nTrack) = 0;
		virtual void Stop(bool bSendSignal = true) = 0;
		virtual void Pause(bool bPause) = 0;
		virtual void Seek(int nTime) = 0;
		virtual void SeekPercentage(float fPercent) = 0;
		virtual int GetTime() const = 0;
		
		bool IsPlaying() const { return m_bPlaying; }
		bool IsPlayComplete() const { return (GetTime() == -1); }

		virtual bool CanPlayFile(const xb::Filename& strFilename) = 0;
	
		virtual bool CheckCapability(std::string& strMessage) = 0;
		
		SigC::Signal0<void> PlayComplete;
};

//------------------------------------------------------------------------------

void signal_play_complete(int n);

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------

#endif

//------------------------------------------------------------------------------
