// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// PropList.cpp
//
// $Id: PropList.cpp,v 1.13 2002/03/09 19:14:08 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "PropList.h"

#include <xb/ErrorManager.h>
#include <xb/Filename.h>

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

PropList::PropList(const std::string& strFilename)
{
	EM::I()->Noise() << "PropList::Init, filename = " << strFilename << endl;
	m_strFilename = strFilename;
	m_Dictionary = PLGetProplistWithPath(m_strFilename.c_str());

	if (m_Dictionary == NULL) {
		throw(xb::file_error("error with proplist: " + strFilename));
	}
	
	int nElts = PLGetNumberOfElements(m_Dictionary);
	EM::I()->Noise() << "number of elts: " << nElts << endl;
}

//------------------------------------------------------------------------------

PropList::~PropList()
{
	PLRelease(m_Dictionary);
}

//------------------------------------------------------------------------------
// merges dictionary at strFilename into existing one
//

void
PropList::Merge(const std::string& strFilename)
{
	m_DictionaryUser = PLGetProplistWithPath(strFilename.c_str());

	if (m_DictionaryUser == NULL) {
		throw(xb::file_error("error with proplist: " + strFilename));
	}

	PLMergeDictionaries(m_Dictionary, m_DictionaryUser);
}

//------------------------------------------------------------------------------

std::string
PropList::GetValue(const std::string& strKey) const
{
	std::string strResult;

	// EM::I()->Trace(std::string("Key: ").append(strKey));
	const char* pKey = strKey.c_str();
	proplist_t val = PLGetDictionaryEntry(m_Dictionary,
										  PLMakeString((char*)pKey));
	if (val != NULL) {
		strResult = PLGetString(val);
	}
	// EM::I()->Trace(std::string("Result: ").append(strResult));
	return strResult;
}

//------------------------------------------------------------------------------

int
PropList::GetIntValue(const std::string& strKey) const
{
	AssertNotNull(m_Dictionary);

	int nResult = 0;

	// EM::I()->Trace(std::string("Key: ").append(strKey));
	const char* pKey = strKey.c_str();
	proplist_t val = PLGetDictionaryEntry(m_Dictionary,
										  PLMakeString((char*)pKey));
	if (val != NULL) {
		std::string strResult = PLGetString(val);
		nResult = atoi(strResult.c_str());
	}
	
	return nResult;
}

//------------------------------------------------------------------------------

bool
PropList::GetBoolValue(const std::string& strKey) const
{
	return (GetIntValue(strKey) == 1);
}

//------------------------------------------------------------------------------

void
PropList::SetValue(const std::string& strKey, const std::string& strValue)
{
	AssertNotNull(m_Dictionary);
	AssertNotNull(m_DictionaryUser);
	
	PLInsertDictionaryEntry(m_DictionaryUser,
							PLMakeString((char *)strKey.c_str()),
							PLMakeString((char *)strValue.c_str()));
	PLInsertDictionaryEntry(m_Dictionary,
							PLMakeString((char *)strKey.c_str()),
							PLMakeString((char *)strValue.c_str()));
}

//------------------------------------------------------------------------------

void
PropList::Save()
{
	AssertNotNull(m_DictionaryUser);

	bool bResult = PLSave(m_DictionaryUser, true);
	if (!bResult) {
		EM::I()->Error() << "Couldn't save user proplist" << endl;
	}
}

//------------------------------------------------------------------------------

void
PropList::CreateUserDict(const std::string& strFilename)
{
	xb::Filename f(strFilename);
	proplist_t key, value, dict;
	key = PLMakeString("config");
	value = PLMakeString("user");
	dict = PLMakeDictionaryFromEntries(key, value, NULL);
	PLSetFilename(dict, PLMakeString((char *)f.c_str()));
	bool bResult = PLSave(dict, true);
	if (!bResult) {
		EM::I()->Error() << "Couldn't create user proplist" << endl;
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------


