// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ThreadRip.cpp
//
// $Id: ThreadRip.cpp,v 1.8 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "ThreadRip.h"

#include <xb/ErrorManager.h>
#include <xb/FileUtil.h>
#include <xb/String.h>

#include "Controller.h"
#include "Info.h"
#include "Library.h"
#include "Output.h"
#include "PlayListManager.h"
#include "PropList.h"
#include "RipperCD.h"
#include "Song.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

ThreadRip::ThreadRip(const std::string& strDevice, const SongList& listSongs,
					 bool bNoAction) :
	Thread(),
	m_strDevice(strDevice),
	m_listSongs(listSongs),
	m_bNoAction(bNoAction)
{
}

//------------------------------------------------------------------------------

ThreadRip::~ThreadRip()
{
}

//------------------------------------------------------------------------------

void
ThreadRip::Run()
{
	m_bActive = true;
	EM::I()->Trace() << "Run: starting rip process..." << endl;

	if (!m_bNoAction) {

		Output* pOutput = System::GetController()->GetOutputPlugin(
			System::GetPropList()->GetValue("Encoder"));
		
		RipperCD ripper;
		try {
			ripper.Init(m_strDevice, pOutput);
		}
		catch(runtime_error e) {
			EM::I()->Error() << e.what() << endl;
			return;
		}

		if (m_listSongs.empty()) {
			cerr << "rip song list empty" << endl;
			return;
		}

		std::string strArtist = m_listSongs[0].GetArtist();
		std::string strAlbum = m_listSongs[0].GetAlbum();
		if (System::GetPropList()->GetValue("CDPlaylistCaps") == "1") {
			strArtist = xb::String::Upper(strArtist);
		}
		std::string strPlayListName = strArtist + " - " + strAlbum;
		EM::I()->Trace() << "playlist name: " << strPlayListName << endl;

		SongList::iterator itr;
		for (itr = m_listSongs.begin(); itr != m_listSongs.end(); ++itr) {
			
			std::string s = "Ripping track " + itr->GetTrack() + ": " +
				itr->GetTitle();
			EM::I()->Trace() << s << endl;
			StatusUpdate(s);
		
			try {
				std::string strFilename =
					ripper.Rip(atoi(itr->GetTrack().c_str()));
				
				std::string strLibFile =
					System::GetLibrary()->Import(*itr, strFilename, false);

				itr->SetFilename(strLibFile);

				Info* pInfo =
					System::GetController()->GetInfoPluginForFile(strLibFile);

				if (pInfo != NULL) {
					pInfo->SetTitle(itr->GetTitle());
					pInfo->SetArtist(itr->GetArtist());
					pInfo->SetAlbum(itr->GetAlbum());
					pInfo->SetTrack(itr->GetTrack());
					pInfo->Save();
				}
				
				xb::FileUtil::Delete(strFilename);
			}
			catch (xb::file_error e) {
				EM::I()->Error() << e.what() << endl;
			}
			catch (invalid_argument e) {
				EM::I()->Error() << e.what() << endl;
			}
		
		}

		System::GetPlayListManager()->NewPlayList(strPlayListName, m_listSongs);

		delete pOutput;
	}
		
	m_bActive = false;
	Done();
	EM::I()->Trace() << "Run: rip done." << endl;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
