// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// WidgetMsgWin.cpp
//
// $Id: WidgetMsgWin.cpp,v 1.19 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include <config.h>
#include "WidgetMsgWin.h"

#include <gtk--/main.h>
#include <gtk--/style.h>
#include <xb/ErrorManager.h>

#include "MWState.h"
#include "Util.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

const int WidgetMsgWin::s_nBackColorRed = 35723;
const int WidgetMsgWin::s_nBackColorGreen = 48573;
const int WidgetMsgWin::s_nBackColorBlue = 34952;

//------------------------------------------------------------------------------

WidgetMsgWin::WidgetMsgWin(int nWidth, int nHeight) :
	Gtk::DrawingArea(),
		 m_nWidth(nWidth),
		 m_nHeight(nHeight),
		 m_pState(NULL)
{
	size(m_nWidth, m_nHeight);
	add_events(GDK_BUTTON_PRESS_MASK);
	button_press_event.connect(SigC::slot(this, &WidgetMsgWin::ButtonPress));
	// button_release_event.connect(SigC::slot(this, &WidgetMsgWin::ButtonPress));
	Gtk::Main::timeout.connect(
		slot(this, &WidgetMsgWin::TimerCallback), 100);
}

//------------------------------------------------------------------------------

WidgetMsgWin::~WidgetMsgWin()
{

}

//------------------------------------------------------------------------------

gint
WidgetMsgWin::expose_event_impl(GdkEventExpose* pEvent)
{
	Draw();

	return 0;					// ???
}

//------------------------------------------------------------------------------

void
WidgetMsgWin::Draw()
{
	m_pixBack.create(get_window(), get_window().width(),
					 get_window().height());

	Gdk_GC gc(m_pixBack);

	SetColor(gc, s_nBackColorRed, s_nBackColorGreen, s_nBackColorBlue);
	
	m_pixBack.draw_rectangle(gc, true, 0, 0, width(), height());
	int nY1 = 0;
	int nY2 = height() - 1;
	m_pixBack.draw_line(this->get_style()->get_black_gc(), 0, nY1, width(), nY1);
	m_pixBack.draw_line(this->get_style()->get_black_gc(), 0, nY2, width(), nY2);
	nY1++;
	nY2--;
	SetColor(gc, 24064, 32768, 23552);
	m_pixBack.draw_line(gc, 0, nY1, width(), nY1);
	m_pixBack.draw_line(gc, 0, nY2, width(), nY2);
	nY1++;
	nY2--;
	SetColor(gc, 27392, 37376, 26880);
	m_pixBack.draw_line(gc, 0, nY1, width(), nY1);
	m_pixBack.draw_line(gc, 0, nY2, width(), nY2);
	nY1++;
	nY2--;
	SetColor(gc, 30208, 40960, 29440);
	m_pixBack.draw_line(gc, 0, nY1, width(), nY1);
	m_pixBack.draw_line(gc, 0, nY2, width(), nY2);
	nY1++;
	nY2--;
	SetColor(gc, 32256, 44032, 31744);
	m_pixBack.draw_line(gc, 0, nY1, width(), nY1);
	m_pixBack.draw_line(gc, 0, nY2, width(), nY2);
	nY1++;
	nY2--;
	SetColor(gc, 34048, 46336, 33280);
	m_pixBack.draw_line(gc, 0, nY1, width(), nY1);
	m_pixBack.draw_line(gc, 0, nY2, width(), nY2);
	nY1++;
	nY2--;
	SetColor(gc, 35072, 47616, 34304);
	m_pixBack.draw_line(gc, 0, nY1, width(), nY1);
	m_pixBack.draw_line(gc, 0, nY2, width(), nY2);
	nY1++;
	nY2--;

	if (m_pState != NULL) {
		m_pState->Draw(&m_pixBack);
	}

	gc.release();
	
	gc.create(get_window());
	get_window().draw_pixmap(gc, m_pixBack, 0, 0, 0, 0);
}

//------------------------------------------------------------------------------

void
WidgetMsgWin::SetColor(Gdk_GC& gc, int nRed, int nGreen, int nBlue)
{
	Gdk_Color color;
	Gdk_Colormap colormap(Gdk_Colormap::get_system());
	color.set_red(nRed);
	color.set_green(nGreen);
	color.set_blue(nBlue);
	colormap.alloc(color);
	gc.set_foreground(color);
}

//------------------------------------------------------------------------------

void
WidgetMsgWin::SetState(MWState* pState)
{
	EM::I()->Trace() << "WidgetMsgWin::SetState(MWState*)" << endl;
	m_pState = pState;
	if (m_pState != NULL) {

		// m_TimerConnection.disconnect();
		
		if (m_pState->NeedsDrawTimer()) {
			/*
			m_TimerConnection = Gtk::Main::timeout.connect(
				slot(this, &WidgetMsgWin::TimerCallback), 100);
			*/
			// Gtk::Main::timeout.connect(
			//	slot(this, &WidgetMsgWin::TimerCallback), 100);
		}
		// else {
		//	m_TimerConnection.disconnect();
		// }
	}

	Draw();
}

//------------------------------------------------------------------------------

int
WidgetMsgWin::TimerCallback()
{
	Draw();
	
	return 1;
}

//------------------------------------------------------------------------------

int
WidgetMsgWin::ButtonPress(GdkEventButton* pEvent)
{
	if (m_pState != NULL) {
		m_pState->ButtonPress(pEvent->x, pEvent->y);
	}
	
	return 0;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
