//------------------------------------------------------------------------------
// main.cpp
// 
// Patrick Crosby
//
// $Id: main.cpp,v 1.26 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------

#if HAVE_CONFIG_H
#  include <config.h>
#endif

#include <dlfcn.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gtk--/main.h>
#include <signal.h>
#include <xb/ErrorManager.h>
#include <xb/Filename.h>
#include <xb/FileUtil.h>

#include "Controller.h"
#include "Interface.h"
#include "Library.h"
#include "Output.h"
#include "Player.h"
#include "PlayListManager.h"
#include "PropList.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

using namespace xtunes;

typedef	Player* (*maker_function)(void);
typedef	Output* (*output_maker_function)(void);

//------------------------------------------------------------------------------

void
signal_shutdown(int n)
{
	EM::I()->Warn() << "got a shutdown signal, cleaning up." << endl;
	Gtk::Main::quit();
}

//------------------------------------------------------------------------------

int
main(int argc, char* argv[])
{

	g_thread_init(NULL);

	EM::I()->SetLogLevel(EM::TRACE);

	EM::I()->Trace() << PACKAGE << " v" << VERSION << endl;

	/*
	std::string strPath = PLUGIN_DIR "/output/liboss.so";
	void* pLib = dlopen(strPath.c_str(), RTLD_NOW);
	if (pLib == NULL) {
		EM::I()->Error("couldn't open " + strPath + ": " + dlerror());
		exit(1);
	}
	
	dlerror();

	output_maker_function mkr = (output_maker_function)dlsym(pLib, "maker");
	const char* pError = dlerror();
	if (pError) {
		EM::I()->Error("couldn't find maker in " + strPath);
		exit(1);
	}

	Output* pOut = mkr();
	*/
	
	try {

//		signal(SIGINT, signal_shutdown);
//		signal(SIGTERM, signal_shutdown);

		PropList dict(PACKAGE_DATA_DIR "/config");
		xb::FileUtil::AssertDirectoryExists(xb::Filename("~/.xtunes/"));
		if (!xb::FileUtil::FileExists(xb::Filename("~/.xtunes/config"))) {
			PropList::CreateUserDict("~/.xtunes/config");
		}

		try {
			dict.Merge("~/.xtunes/config");
		}
		catch(xb::file_error e) {
			EM::I()->Error() << "error in ~/.xtunes/config, proceeding.." <<
				endl;
		}
		System::SetPropList(&dict);
	
		EM::I()->Trace() << dict.GetValue("Library") << endl;

		Library library(dict.GetValue("Library"));
		System::SetLibrary(&library);
		// library.Dump();
	
		PlayListManager* pManager =
			new PlayListManager(dict.GetValue("PlayListRoot"));
		System::SetPlayListManager(pManager);

		Gtk::Main kit(argc, argv);

		gtk_rc_parse_string("pixmap_path \"" INTERFACE_DATA_DIR "\"");
		gtk_rc_parse(INTERFACE_DATA_DIR "/xtunes.gtkrc");

		Controller controller;
		System::SetController(&controller);
		
		Interface interface;
		interface.Init();
		interface.get_window().set_decorations((GdkWMDecoration)0);

		gtk_rc_parse_string("pixmap_path \"" INTERFACE_DATA_DIR "\"");
		gtk_rc_parse(INTERFACE_DATA_DIR "/xtunes.gtkrc");
		
		gdk_threads_enter();
		kit.run();
		gdk_threads_leave();
	}
	catch (logic_error e) {
		EM::I()->Error() << "Logic Exception caught:  " << e.what() << endl;
		// EM::I()->Dialog(std::string("Exception caught:  ") + e.what());
	}
	catch (runtime_error e) {
		EM::I()->Error() << "Runtime Exception caught:  " << e.what() << endl;
		// EM::I()->Dialog(std::string("Exception caught:  ") +
		//			e.what());
	}

	return 0;
}

//------------------------------------------------------------------------------
